/***
 *  Enumerating, up to isomorphism, all the genus 3 hyperelliptic curves
 *  defined over a finite field.
 *
 *  Distributed under the terms of the GNU Lesser General Public License (L-GPL)
 *                  http://www.gnu.org/licenses/
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  Copyright 2011, R. Lercier & C. Ritzenthaler
 */

 /*

  This program enumerates all the isomorphic classes of gneus 3 hyperelliptic
  models modulo some prime p.


  Three possible uses:

     _ If you type "magma p:=11 countenum.m", the script enumerates modulo 11
     (or p = any other prime, up to you) all the posible shioda invariants
     modulo p, and read from them the geometric automorphism
     groups. Statistics are regularly printed.

     _ If you type "magma p:=11 reconstruct:=true twists:=true countenum.m",
     the script enumerates modulo 11 (or p = any other prime, up to you) all
     the posible shioda invariants modulo p, and reconstruct from them a model
     for the twists that share these invariants. Statistics are regularly
     printed. Records of the form
                        1. Shioda Invariants
                        2. Automorphism group
                        3. Singular curve or not
                        4. Number of models
                        5. Models
     for all the curves are stored in the file CurvesMod_11.dat.gz.
     See the functions WriteCurves() and ReadCurves() for a precise description
     of the format of these records.

     _ If you type "magma p:=11 reload:=true countenum.m", the script
     read the file CurvesMod_11.dat.gz (or p = any other prime, up to you),
     one record by one record. Statistics are regularly printed too

     Databases "CurvesMod_p.dat.gz" of curves for all the primes p between 11
     and 47 have been computed by the authors and are available on request.
 */
 
/* Is p defined on the command line ? */
if not assigned(p) then
  "Usage: \"magma p:=11 [reconstruct:=true] [reload:=true] countenum.m\""; quit;
end if;

/* Defaults */
SetHistorySize(0);
if not assigned(reconstruct) then reconstruct := false; end if;
if not assigned(reload) then reload := false; end if;
if not assigned(twists) then twists := false; end if;

/* Convert arguments provided on the command line */
if Type(p) eq MonStgElt then
    p := StringToInteger(p);
end if;

if Type(reconstruct) eq MonStgElt and reconstruct eq "true" then
    reconstruct := true;
else
    reconstruct := false;
end if;

if Type(reload) eq MonStgElt and reload eq "true" then
    reload := true;
else
    reload := false;
end if;

if Type(twists) eq MonStgElt and twists eq "true" then
    twists := true;
else
    twists := false;
end if;

if reconstruct and reload then
    "reconstruct and reload can not be both set to true, I give up";
    quit;
end if;

/* Loading g3twists */
AttachSpec("package/spec");
    
function NbTwists(q)

    return
	  2 * q^5
	+ 2 * q^3
	- 2
	- 2 * ( ((q+1) mod 4 eq 0)                    select q^2-q else 0 )
	+ 2 * ( ((q mod 2) ne 0) and ((q mod 3) ne 0) select q-1   else 0 )
	+ ( ((q-1) mod 8 eq 0)                        select 4     else 0 )
	+ ( ((q-1) mod 7 eq 0)                        select 12    else 0 )
	+ ( (q mod 7 eq 0)                            select 2     else 0 )
	+ ( ((q mod 12) in {1, 5})                    select 2     else 0 );

end function;

/* Some tools */
procedure PrintStats(tm, nbfree, nball, S8, D12, C14, C2D8, C2, U6, V8, C4, D4, C2S4, C2C4, C2p3)

    m := Degree(GF(p));
    
    "";
    "************";
    "*", nbfree, "/", p^5+p^4+p^3+p^2+p+2, " inv. tested in", Cputime(tm), "s";
    "";

    TotGen3 := S8[1] + D12[1] + C14[1] + C2D8[1] + C2[1] + U6[1] + V8[1] + C4[1] + D4[1] + C2S4[1] + C2C4[1] + C2p3[1];
    TotTwists := S8[3] + D12[3] + C14[3] + C2D8[3] + C2[3] + U6[3] + V8[3] + C4[3] + D4[3] + C2S4[3] + C2C4[3] + C2p3[3];
    TotSing := S8[2] + D12[2] + C14[2] + C2D8[2] + C2[2] + U6[2] + V8[2] + C4[2] + D4[2] + C2S4[2] + C2C4[2] + C2p3[2];

    "  ", nbfree, "free invariants J2, J3, ..., J7 obtained,", p^5+p^4+p^3+p^2+p+2, "awaited";
    "  ", nball,  "invariants J2, J3, ..., J7, J8, J9, J10 obtained";
    "  ", TotGen3,"curves found up to some geo. isomorphism,", p^5, "awaited";
    "  ", TotTwists+TotGen3,"curves found up to some arith. isomorphism,", NbTwists(p), "awaited";
    "";

    "Automorphism group statistics are:";"";
    printf "      & C2    & D4   & C4   & C2p3  & C2C4    & D12    & C2D8    & C14    & U6    & V8  & C2S4  & S8  & Total\\\\\n";
    printf "Gen 3 & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o\\\\\n", 
	C2[1],  D4[1],  C4[1],  C2p3[1],  C2C4[1],  D12[1], C2D8[1],  C14[1],  U6[1], V8[1],  C2S4[1],  S8[1], TotGen3;
    printf "Twist & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o\\\\\n", 
	C2[3],  D4[3],  C4[3],  C2p3[3],  C2C4[3],  D12[3], C2D8[3],  C14[3],  U6[3], V8[3],  C2S4[3],  S8[3], TotTwists;
    printf "Sing. & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o\\\\\n", 
	C2[2],  D4[2],  C4[2],  C2p3[2],  C2C4[2],  D12[2], C2D8[2],  C14[2],  U6[2],  V8[2],  C2S4[2],  S8[2], TotSing;
    
    printf "Total & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o    & %o\\\\\n",
	&+C2,  &+D4,  &+C4,  &+C2p3,  &+C2C4,  &+D12, &+C2D8,  &+C14,  &+U6, &+V8,  &+C2S4,  &+S8, TotGen3+TotTwists+TotSing;

end procedure;

forward WriteFFSequence;
procedure WriteFFSequence(F, S)
    U := Universe(S);
    if IsPrimeField(U) then
	for s in S do Put(F, Sprintf("%o ", s)); end for; Puts(F, "");
    else
	for s in S do
	    WriteFFSequence(F, Eltseq(s));
	end for;
    end if;
end procedure;

forward ReadFFSequence;
function ReadFFSequence(F, FF, L)
    if IsPrimeField(FF) then
	str := Gets(F);
	if IsEof(str) then return []; end if;
	S := [FF!c : c in StringToIntegerSequence(str)];
    else
	S := [];
	for i := 1 to L do
	    Append(~S, ReadFFSequence(F, BaseField(FF), Degree(FF, BaseField(FF))));
	end for;
    end if;
    return S;
end function;

procedure WriteCurves(F, CC)

    J, IG, singular, Hs := Explode(CC);

    WriteFFSequence(F, J);
    Puts(F, Sprintf("%o %o", IG[1], IG[2]));
    Puts(F, singular select "1" else "0");
    Puts(F, Sprintf("%o", #Hs));
    for H in Hs do
	WriteFFSequence(F, [Coefficient(H, i) : i in [0..8]]);
    end for;
    
    Puts(F, "");
    Flush(F);

end procedure;

function ReadCurves(F, FFx)

    
    FF := CoefficientRing(FFx);
    
    J := ReadFFSequence(F, FF, 9);
    if #J eq 0 then return []; end if;

    str := Gets(F); IG := StringToIntegerSequence(str); IG := <IG[1], IG[2]>;
    str := Gets(F); singular := StringToInteger(str) eq 1 select true else false;
    str := Gets(F); nbH := StringToInteger(str);
    Hs := []; for i := 1 to nbH do 
	H := FFx!ReadFFSequence(F, FF, 9);
	Append(~Hs, H);
    end for;    
    str := Gets(F);

    return [* J, IG, singular, Hs *];

end function;

/* Let's go */
""; 
if reconstruct then
    "Enumerating curves modulo p = ", p;
    InputFile := false;
    OutputFile := POpen("gzip > CurvesMod_" cat IntegerToString(p) cat ".dat.gz", "w");
elif reload then
    "Reading curves modulo p = ", p;
    InputFile := POpen("gzip -d < CurvesMod_" cat IntegerToString(p) cat ".dat.gz", "r");
    OutputFile := false;
else
    "Counting curves modulo p = ", p;
    InputFile := false;
    OutputFile := false;
end if;

FF := GF(p); FFx<x> := PolynomialRing(FF);

if not reload then
    WPSCtxt := WPSEnumInit(FF, [2, 3, 4, 5, 6, 7]);
    WPSEnumNext(~FJI, ~WPSCtxt); 
else
    FJI := 0;
end if;
FJI0 := FJI;

S8 := [0, 0, 0]; D12 := [0, 0, 0]; C14 := [0, 0, 0]; C2D8 := [0, 0, 0]; C2 := [0, 0, 0]; U6 := [0, 0, 0];
V8 := [0, 0, 0]; C4 := [0, 0, 0]; D4 := [0, 0, 0]; C2S4 := [0, 0, 0]; C2C4 := [0, 0, 0]; C2p3 := [0, 0, 0];

nbfree := 0; nball := 0; nbtrue := 0; nbcurve := 0; tm := Cputime();
repeat

    nbfree +:= 1;

    if (nbfree mod 20000 eq 0) or
	(reconstruct eq true and nbfree mod 1000 eq 0) then
	PrintStats(tm, nbfree, nball, S8, D12, C14, C2D8, C2, U6, V8, C4, D4, C2S4, C2C4, C2p3);
    end if;


    if reload then
	JIs := [0];
    else
	JIs := ShiodaAlgebraicInvariants(FJI : ratsolve := true);
    end if;
    nball +:= #JIs;

    for _J in JIs do

	if reconstruct then
	    J := _J;
	    singular := DiscriminantFromShiodaInvariants(J) eq 0;

	    if singular or not twists then
		H, G := HyperellipticPolynomialsFromShiodaInvariants(J);
	    else
		H, G := TwistedHyperellipticPolynomialsFromShiodaInvariants(J);
	    end if;
	    if Type(H[1]) eq BoolElt then
		printf "Error at J = %o (none curve found)\n", J;
		continue;
	    end if;

	    HJ := ShiodaInvariants(H[1] : normalize := true);
	    if ChangeUniverse(J, Universe(HJ)) ne HJ then
		printf "Error at J = %o, HJ = %o  (reconstruction)\n", J, HJ;
		continue;
	    end if;
	
	    if &and[ &and[c in GF(p) : c in Eltseq(h)] : h in H] ne true then
		printf "Error at J = %o (none rational model)\n", J;
		continue;
	    end if;
		    
	    IG :=  IdentifyGroup(G);
	    WriteCurves(OutputFile, [* J, IG, singular, H *]);

	elif reload then

	    CC := ReadCurves(InputFile, FFx);
	    if #CC eq 0 then
		FJI := 0; nball -:= 1; nbfree -:= 1; break;
	    else
		FJI := 1;
		J, IG, singular, H := Explode(CC);
	    end if;

	else
	    J := _J; H := [0];
	    singular := DiscriminantFromShiodaInvariants(J) eq 0;
	    
	    G := GeometricAutomorphismGroupFromShiodaInvariants(J);
	    IG :=  IdentifyGroup(G);
	    
	end if;
	    
	if not singular then idx := 1; else idx := 2; end if;

	case IG:	
	    when <1, 1>:
		S8[idx] +:= 1;
	    when <12, 4>:
		D12[idx] +:= 1;
	    when <14, 2>:
		C14[idx] +:= 1;
	    when <16, 11>:
		C2D8[idx] +:= 1;
	    when <2, 1>:
		C2[idx] +:= 1;
	    when <24, 5>:
		U6[idx] +:= 1;
	    when <32, 9>:
		V8[idx] +:= 1;
	    when <4, 1>:
		C4[idx] +:= 1;
	    when <4, 2>:
		D4[idx] +:= 1;
	    when <48, 48>:
		C2S4[idx] +:= 1;
	    when <8, 2>:
		C2C4[idx] +:= 1;
	    when <8, 5>:
		C2p3[idx] +:= 1;
	    end case;
		
	    case IG:	
	    when <1, 1>:
		S8[3] +:= #H-1;
	    when <12, 4>:
		D12[3] +:= #H-1;
	    when <14, 2>:
		C14[3] +:= #H-1;
	    when <16, 11>:
		C2D8[3] +:= #H-1;
	    when <2, 1>:
		C2[3] +:= #H-1;
	    when <24, 5>:
		U6[3] +:= #H-1;
	    when <32, 9>:
		V8[3] +:= #H-1;
	    when <4, 1>:
		C4[3] +:= #H-1;
	    when <4, 2>:
		D4[3] +:= #H-1;
	    when <48, 48>:
		C2S4[3] +:= #H-1;
	    when <8, 2>:
		C2C4[3] +:= #H-1;
	    when <8, 5>:
		C2p3[3] +:= #H-1;
	    end case;
	    
	end for;

	
	if not reload then
	    WPSEnumNext(~FJI, ~WPSCtxt);
	end if;
	
until FJI eq FJI0;

if reconstruct then
    delete OutputFile;
elif reload then
    delete InputFile;
end if;

PrintStats(tm, nbfree, nball, S8, D12, C14, C2D8, C2, U6, V8, C4, D4, C2S4, C2C4, C2p3);

quit;
