/***
 *  Proof checker of the reconstruction lemmas.
 *
 *  Distributed under the terms of the GNU Lesser General Public License (L-GPL)
 *                  http://www.gnu.org/licenses/
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  Copyright 2011, R. Lercier & C. Ritzenthaler
 */

 /*

   This program checks that the reconsruction lemmas of [LeRi2012] are valid.

   Bibliography.
  
  [LeRi2012] "Hyperelliptic curves and their invariants: geometric, arithmetic
  and algorithmic aspects", Journal of Algebra, Elsevier, 2012. To appear.
  Available at http://dx.doi.org/10.1016/j.jalgebra.2012.07.054

 */
 
AttachSpec("package/spec");

/* Set CheckInDepth to true for all tests, but beware, it takes ages... */
CheckInDepth := false;

 "Proof checker of the reconstruction Lemmas stated in the paper";
"  \"Reconstruction of hyperelliptic curves of genus 3\",";
"  R. Lercier, C. Ritzenthaler, 2011.";
"";

Pa<a8, a0, a7, a1, a6, a2, a5, a3, a4> := FunctionField(RationalField(), 9);
Px<x> := PolynomialRing(Pa);

PJ<J10, J9, J8, J7, J6, J5, J4, J3, J2 > := PolynomialRing(RationalField(), [10, 9, 8, 7, 6, 5, 4, 3, 2]);

load "genus3dets.m";
load "D4LemmaConverse.m";
"";

/* Shioda relations */
Relations := [Evaluate(EQ, [J8, J9, J10]) :
    EQ in ShiodaAlgebraicInvariants([J2, J3, J4, J5, J6, J7] : ratsolve := false)];

function IsLemmaConverseTrue(Stratum : Group := <>)

    IJ := ideal<PJ | Stratum>;

    QJ := quo<PJ | IJ>;

    FQJ := RingOfFractions(QJ);    
    j10, j9, j8, j7, j6, j5, j4, j3, j2 := Explode([FQJ.i : i in [1..Rank(FQJ)]]);

    H, G := HyperellipticPolynomialsFromShiodaInvariants([j2, j3, j4, j5, j6, j7, j8, j9, j10]);

    TI := ShiodaInvariants(H[1]);

    test := ShiodaInvariantsEqual(
	[FQJ!j : j in TI],
	[FQJ!j : j in [j2, j3, j4, j5, j6, j7, j8, j9, j10]]
	);

    if  #Group ne 0 then
	test and:= IdentifyGroup(G) eq Group;
    end if;

    return test ;

end function;

/* A direct (and faster) check for C2^3 */
function IsC2p3LemmaConverseTrue(Stratum : Group := <>)

    IJ := ideal<PJ | Stratum>;

    QJ := quo<PJ | IJ>;

    FQJ := RingOfFractions(QJ);
    j10, j9, j8, j7, j6, j5, j4, j3, j2 := Explode([FQJ.i : i in [1..Rank(FQJ)]]);

    H, G := HyperellipticPolynomialsFromShiodaInvariants([j2, j3, j4, j5, j6, j7, j8, j9, j10]);

    f := H[1]; CR := CoefficientRing(f);
    
    a0 := Coefficient(f, 0);
    a2 := Coefficient(f, 2);
    a4 := Coefficient(f, 4);
    a6 := Coefficient(f, 6);
    a8 := Coefficient(f, 8);

    /* Only j2 to j5 need to be checked 
       since j6, ..., j10 are rational functions of j2,..,j5,
       (j5 is algebraic of degree 5),
    */
    test :=
	/* j2 */
    (2*a8*a0 + 1/14*a6*a2 + 1/70*a4^2 eq CR!j2) and
	/* j3 */
    (3/35*a8*a0*a4 + 9/392*a8*a2^2 + 9/392*a0*a6^2 - 33/13720*a6*a2*a4 +
	9/34300*a4^3 eq CR!j3) and
	/* j4 */
    (2/3*a8^2*a0^2 - 1/294*a8*a0*a6*a2 - 1/147*a8*a0*a4^2 + 3/1372*a8*a2^2*a4 + 
	3/1372*a0*a6^2*a4 + 1/14406*a6^2*a2^2 + 31/57624*a6*a2*a4^2 +
	1/57624*a4^4 eq CR!j4) and
	/* j5 */
    (2/35*a8^2*a0^2*a4 + 3/196*a8^2*a0*a2^2 + 3/196*a8*a0^2*a6^2 + 
	53/6860*a8*a0*a6*a2*a4 - 1/1715*a8*a0*a4^3 - 1/9604*a8*a6*a2^3 + 
	41/192080*a8*a2^2*a4^2 - 1/9604*a0*a6^3*a2 + 41/192080*a0*a6^2*a4^2 - 
	13/336140*a6^2*a2^2*a4 - 13/1344560*a6*a2*a4^3 + 1/672280*a4^5 eq CR!j5);

    if  #Group ne 0 then
	test and:= IdentifyGroup(G) eq Group;
    end if;

    return test ;

end function;

function IsLemmaDirectTrue(Stratum, model)
    rJI := Reverse(ShiodaInvariants(model));
    test := &and[Evaluate(E, rJI) eq 0 : E in Stratum];

    return test;
end function;

if CheckInDepth then
    "All tests are going to be done";
    "  (but be patient, some tests take ages (days...).";
else
    "Heavy tests are going to be skipped";
    "  (rerun with \"CheckInDepth\" set to \"true\" for all).";
end if;
"";

/* C2xS4
 *******/
C2S4Stratum := [
    J10,
    J9,
    J8,
    J7,
    J6,
    J5,
    J4,
    30*J3^2 - J2^3
    ];
fC2S4 := x^8 + 14*x^4 + 1;

"_ C2xS4 Lemma.";
"           Direct implication       :", IsLemmaDirectTrue(C2S4Stratum, fC2S4);
"           Converse implication     :", IsLemmaConverseTrue(C2S4Stratum : Group := <48, 48>);
"";

/* V8
 ****/
V8Stratum := [
    2520*J10 - J2^5,
    J9,
    420*J8 + J2^4,
    J7,
    36*J6 + J2^3,
    J5,
    6*J4 - J2^2,
    J3
];
fV8 := x^8 - 1;

"_ V8 Lemma.";
"           Direct implication       :", IsLemmaDirectTrue(V8Stratum, fV8);
"           Converse implication     :", IsLemmaConverseTrue(V8Stratum : Group := <32, 9>);
"";

/* U6
 ****/
U6Stratum := [
    430080*J10 + J2^5,
    J9,
    17920*J8 - J2^4,
    J7,
    2304*J6 + J2^3,
    J5,
    96*J4 - J2^2,
    J3
];
fU6 := x*(x^6-1);

"_ U6 Lemma.";
"           Direct implication       :", IsLemmaDirectTrue(U6Stratum, fU6);
"           Converse implication     :", IsLemmaConverseTrue(U6Stratum : Group := <24, 5>);
"";

/* C14
 ****/
C14Stratum := [
    J10,
    J9,
    J8,
    J6,
    J5,
    J4,
    J3,
    J2
];
fC14 := x^7-1;

"_ C14 Lemma.";
"           Direct implication       :", IsLemmaDirectTrue(C14Stratum, fC14);
"           Converse implication     :", IsLemmaConverseTrue(C14Stratum : Group := <14, 2>);
"";

/* C2xD8
 *******/
C2D8Stratum := [
    630*J10 - 15*J4^2*J2 - 30*J4*J3^2 + J4*J2^3,
    18*J9 - 2*J5*J2^2 + 15*J4*J3*J2 + 60*J3^3 - 2*J3*J2^3,
    1260*J8 - 18*J4^2 - 63*J4*J2^2 - 420*J3^2*J2 + 14*J2^4,
    3*J7 - J5*J2 + 3*J4*J3,
    18*J6 - 9*J4*J2 - 60*J3^2 + 2*J2^3,
    5*J5^2 - 6*J4^2*J2 - 30*J4*J3^2 + J4*J2^3,
    3*J5*J4 - 2*J5*J2^2 + 15*J4*J3*J2 + 60*J3^3 - 2*J3*J2^3,
    60*J5*J3 + 18*J4^2 - 15*J4*J2^2 - 60*J3^2*J2 + 2*J2^4,
    54*J4^3 - 81*J4^2*J2^2 - 1080*J4*J3^2*J2 + 36*J4*J2^4 - 3600*J3^4 + 
        240*J3^2*J2^3 - 4*J2^6
];
fC2D8 := x^8 + a4*x^4 +a0;

"_ C2xD8 Lemma."; "";
"           Direct implication       :", IsLemmaDirectTrue(C2D8Stratum, fC2D8);
"";
"           Case -30*J3^2+J2^3 <> 0 ";
"           Converse implication     :", IsLemmaConverseTrue(C2D8Stratum : Group := <16, 11>);
"";
"           Case -30*J3^2+J2^3 = 0 but J4 <> 0";
"           Converse implication     :", IsLemmaConverseTrue(C2D8Stratum cat [-30*J3^2+J2^3, 2*J4-3*J2^2] : Group := <16, 11>);
"";
"           Case -30*J3^2+J2^3 = 0 and J4 = 0";
"           C2xS4 stratum            :", IsLemmaConverseTrue(C2D8Stratum cat [-30*J3^2+J2^3, J4] : Group := <48, 48>);
"";


/* D12
 *****/
D12Stratum := [
    30240*J10 - 600*J4^2*J2 - 390*J4*J3^2 + 13*J4*J2^3,
    3456*J9 - 24*J5*J2^2 - 120*J4*J3*J2 - 30*J3^3 + J3*J2^3,
    60480*J8 - 14976*J4^2 + 504*J4*J2^2 + 210*J3^2*J2 - 7*J2^4,
    12*J7 - J5*J2 - 2*J4*J3,
    576*J6 - 72*J4*J2 - 30*J3^2 + J2^3,
    180*J5^2 - 96*J4^2*J2 - 30*J4*J3^2 + J4*J2^3,
    576*J5*J4 - 24*J5*J2^2 - 120*J4*J3*J2 - 30*J3^3 + J3*J2^3,
    720*J5*J3 - 2304*J4^2 + 120*J4*J2^2 + 30*J3^2*J2 - J2^4,
    55296*J4^3 - 5184*J4^2*J2^2 - 4320*J4*J3^2*J2 + 144*J4*J2^4 - 900*J3^4 + 
        60*J3^2*J2^3 - J2^6
];
fD12 := x * (x^6 + a4*x^3 + a1);

"_ D12 Lemma."; "";
"           Direct implication       :", IsLemmaDirectTrue(D12Stratum, fD12);
"";
"           Case -30*J3^2+J2^3 <> 0 ";
"           Converse implication     :", IsLemmaConverseTrue(D12Stratum : Group := <12, 4>);
"";
"           Case -30*J3^2+J2^3 = 0 but J4 <> 0";
"           Converse implication     :", IsLemmaConverseTrue(D12Stratum cat [-30*J3^2+J2^3, 32*J4-3*J2^2] : Group := <12, 4>);
"";
"           Case -30*J3^2+J2^3 = 0 and J4 = 0";
"           C2xS4 stratum            :", IsLemmaConverseTrue(D12Stratum cat [-30*J3^2+J2^3, J4] : Group := <48, 48>);
"";

/* C2xC4
 *******/
C2C4Stratum := [
    2041200*J10 + 560520*J6*J4 + 2205*J6*J2^2 + 98091*J4^2*J2 - 5607*J4*J2^3 +  56*J2^5,
    J9,
    85050*J8 + 13860*J6*J2 + 37908*J4^2 - 2961*J4*J2^2 + 28*J2^4,
    J7,
    648000*J6^2 - 356400*J6*J4*J2 + 4275*J6*J2^3 - 839808*J4^3 + 75249*J4^2*J2^2 - 1449*J4*J2^4 + 8*J2^6,
    J5,
    J3
];
fC2C4 := (x^4 - 1) * (x^4 + a2 * x^2 + 1);

"_ C2xC4 Lemma."; "";
"           Direct implication       :", IsLemmaDirectTrue(C2C4Stratum, fC2C4);

"";
"           Case 6*J4-J2^2 <> 0 ";
"           Converse implication     :", IsLemmaConverseTrue(C2C4Stratum : Group := <8, 2>);
"";

"           Case 6*J4-J2^2 = 0 but J6 + 1/36*J2^3 <> 0";
"           Converse implication     :", IsLemmaConverseTrue(C2C4Stratum cat [6*J4-J2^2, J6 - 65/576*J2^3] : Group := <8, 2>);
"";

"           Case 6*J4-J2^2 = 0 and J6 + 1/36*J2^3 = 0";
"           V8 stratum               :", IsLemmaConverseTrue(C2C4Stratum cat [6*J4-J2^2, J6 + 1/36*J2^3] : Group := <32, 9>);
"";


"           Case 96*J4-J2^2 = 0 ";
"           U6 stratum               :", IsLemmaConverseTrue(C2C4Stratum cat [96*J4-J2^2, J6 + 1/2304*J2^3] : Group := <24,5>);
"";

"           Case 147*J4-2*J2^2 = 0 and J6 - 2/3087*J2^3 <> 0 ";
"           Converse implication     :", IsLemmaConverseTrue(C2C4Stratum cat [147*J4-2*J2^2, J6 - 47/197568*J2^3] : Group := <8, 2>);
"";

"           Case 147*J4-2*J2^2 = 0 and J6 - 2/3087*J2^3 = 0 ";
"           Converse implication     :", IsLemmaConverseTrue(C2C4Stratum cat [147*J4-2*J2^2, J6 - 2/3087*J2^3] : Group := <8, 2>);
"";



/* C2^3
 *******/
C2p3Stratum := [
    170100*J10 + 37890*J6*J4 - 22680*J5^2 + 4221*J4^2*J2 + 1680*J4*J3^2 - 56*J4*J2^3,
    9720*J9 + 810*J7*J2 - 3330*J6*J3 - 2187*J5*J4 + 108*J5*J2^2 - 360*J4*J3*J2 - 240*J3^3 + 8*J3*J2^3,
    85050*J8 - 11655*J6*J2 + 11340*J5*J3 + 2187*J4^2 - 1260*J4*J2^2 - 840*J3^2*J2 + 28*J2^4,
    546750*J7^2 - 963900*J6*J4^2 + 453600*J6*J4*J2^2 + 1404000*J6*J3^2*J2 - 46800*J6*J2^4 + 320760*J5^2*J4 - 196830*J5^2*J2^2 + 
        204120*J5*J4*J3*J2 - 680400*J5*J3^3 + 22680*J5*J3*J2^3 - 12312*J4^3*J2 + 537570*J4^2*J3^2 + 32301*J4^2*J2^3 + 
        157680*J4*J3^2*J2^2 - 5256*J4*J2^5 + 82800*J3^4*J2 - 5520*J3^2*J2^4 + 92*J2^7,
    145800*J7*J6 + 4050*J7*J2^3 - 64800*J6*J5*J2 + 83700*J6*J4*J3 - 16650*J6*J3*J2^2 + 77760*J5^2*J3 + 13122*J5*J4^2 - 
        5751*J5*J4*J2^2 + 8640*J5*J3^2*J2 + 252*J5*J2^4 - 10260*J4^2*J3*J2 + 2880*J4*J3^3 - 1896*J4*J3*J2^3 - 1200*J3^3*J2^2 + 
        40*J3*J2^5,
    18225*J7*J5 + 7425*J6*J4*J2 + 43200*J6*J3^2 - 1440*J6*J2^3 - 10935*J5^2*J2 + 42525*J5*J4*J3 + 5103*J4^3 - 675*J4^2*J2^2 + 
        2250*J4*J3^2*J2 - 75*J4*J2^4 + 1800*J3^4 - 120*J3^2*J2^3 + 2*J2^6,
    4860*J7*J4 - 810*J7*J2^2 - 1080*J6*J5 + 3330*J6*J3*J2 - 837*J5*J4*J2 - 2880*J5*J3^2 - 12*J5*J2^3 + 2916*J4^2*J3 + 
        360*J4*J3*J2^2 + 240*J3^3*J2 - 8*J3*J2^4,
    12150*J7*J3 + 7560*J6*J4 - 1665*J6*J2^2 - 2430*J5^2 + 1620*J5*J3*J2 + 837*J4^2*J2 + 1530*J4*J3^2 - 231*J4*J2^3 - 120*J3^2*J2^2 
        + 4*J2^5,
    32400*J6^2 + 28350*J6*J4*J2 + 87750*J6*J3^2 - 2925*J6*J2^3 - 21870*J5^2*J2 + 76545*J5*J4*J3 + 13122*J4^3 - 405*J4^2*J2^2 + 
        5130*J4*J3^2*J2 - 171*J4*J2^4 + 3600*J3^4 - 240*J3^2*J2^3 + 4*J2^6,
    194400*J6*J5^2 + 2034720*J6*J4^2*J2 + 2073600*J6*J4*J3^2 - 777870*J6*J4*J2^3 - 2193750*J6*J3^2*J2^2 + 73125*J6*J2^5 - 
        913680*J5^2*J4*J2 + 518400*J5^2*J3^2 + 179550*J5^2*J2^3 + 1516320*J5*J4^2*J3 + 419175*J5*J4*J3*J2^2 + 1555200*J5*J3^3*J2 - 
        51840*J5*J3*J2^4 + 244944*J4^4 + 112590*J4^3*J2^2 - 1170720*J4^2*J3^2*J2 - 80451*J4^2*J2^4 + 86400*J4*J3^4 - 
        306810*J4*J3^2*J2^3 + 10131*J4*J2^6 - 147600*J3^4*J2^2 + 9840*J3^2*J2^5 - 164*J2^8,
    22680*J6*J5*J4 - 4995*J6*J5*J2^2 - 14850*J6*J4*J3*J2 - 86400*J6*J3^3 + 2880*J6*J3*J2^3 - 7290*J5^3 + 26730*J5^2*J3*J2 + 
        2511*J5*J4^2*J2 - 80460*J5*J4*J3^2 - 693*J5*J4*J2^3 - 360*J5*J3^2*J2^2 + 12*J5*J2^5 - 10206*J4^3*J3 + 1350*J4^2*J3*J2^2 - 
        4500*J4*J3^3*J2 + 150*J4*J3*J2^4 - 3600*J3^5 + 240*J3^3*J2^3 - 4*J3*J2^6,
    16200*J6*J5*J3 + 45360*J6*J4^2 - 17550*J6*J4*J2^2 - 49950*J6*J3^2*J2 + 1665*J6*J2^4 - 14580*J5^2*J4 + 2430*J5^2*J2^2 + 
        22275*J5*J4*J3*J2 + 43200*J5*J3^3 - 1440*J5*J3*J2^3 + 5022*J4^3*J2 - 34560*J4^2*J3^2 - 2223*J4^2*J2^3 - 7650*J4*J3^2*J2^2 +
        255*J4*J2^5 - 3600*J3^4*J2 + 240*J3^2*J2^4 - 4*J2^7,
    7620480*J6*J4^3 - 4626720*J6*J4^2*J2^2 - 6609600*J6*J4*J3^2*J2 + 929070*J6*J4*J2^4 + 10368000*J6*J3^4 + 1502550*J6*J3^2*J2^3 - 
        61605*J6*J2^6 + 874800*J5^3*J3 - 2449440*J5^2*J4^2 + 947700*J5^2*J4*J2^2 - 3207600*J5^2*J3^2*J2 - 89910*J5^2*J2^4 + 
        3440880*J5*J4^2*J3*J2 + 16912800*J5*J4*J3^3 - 982935*J5*J4*J3*J2^3 - 1555200*J5*J3^3*J2^2 + 51840*J5*J3*J2^5 + 
        843696*J4^4*J2 - 4581360*J4^3*J3^2 - 559278*J4^3*J2^3 - 168480*J4^2*J3^2*J2^2 + 125091*J4^2*J2^5 - 64800*J4*J3^4*J2 + 
        305370*J4*J3^2*J2^4 - 10107*J4*J2^7 + 432000*J3^6 + 104400*J3^4*J2^3 - 8400*J3^2*J2^6 + 148*J2^9,
    97200*J5^4 - 656100*J5^3*J3*J2 - 437400*J5^2*J4^2*J2 + 2916000*J5^2*J4*J3^2 + 212625*J5^2*J4*J2^3 + 753300*J5^2*J3^2*J2^2 - 
        25110*J5^2*J2^5 + 2494800*J5*J4^3*J3 - 1336500*J5*J4^2*J3*J2^2 - 5427000*J5*J4*J3^3*J2 + 180900*J5*J4*J3*J2^4 - 
        3024000*J5*J3^5 + 201600*J5*J3^3*J2^3 - 3360*J5*J3*J2^6 + 381024*J4^5 - 259200*J4^4*J2^2 - 186300*J4^3*J3^2*J2 + 
        66960*J4^3*J2^4 + 2592000*J4^2*J3^4 + 151200*J4^2*J3^2*J2^3 - 7920*J4^2*J2^6 + 351000*J4*J3^4*J2^2 - 23400*J4*J3^2*J2^5 + 
        390*J4*J2^8 + 108000*J3^6*J2 - 10800*J3^4*J2^4 + 360*J3^2*J2^7 - 4*J2^10
];
fC2p3 := x^8 + a2*x^6 + a4*x^4 + a2*x^2 + 1;

"_ C2^3 Lemma."; "";
"           Direct implication       :", IsLemmaDirectTrue(C2p3Stratum, fC2p3);

"";
"           Case -18*J6+9*J4*J2+60*J3^2-2*J2^3 <> 0 ";
"           Converse implication     :", IsC2p3LemmaConverseTrue(C2p3Stratum : Group := <8, 5>);
"";

"           Case -18*J6+9*J4*J2+60*J3^2-2*J2^3 = 0";
"           C2xD8 stratum            :", IsLemmaConverseTrue(C2p3Stratum cat [-18*J6+9*J4*J2+60*J3^2-2*J2^3, J4^3 - 3/2*J4^2*J2^2 - 20*J4*J3^2*J2 + 2/3*J4*J2^4 - 200/3*J3^4 + 40/9*J3^2*J2^3 - 2/27*J2^6] : Group := <16, 11>);
"";

/* C4
 *****/
C4Stratum := [
    4800902400*J10^2 + 262906560*J10*J4^2*J2 - 9485910*J10*J4*J2^3 + 
        92610*J10*J2^5 - 3541737150*J8^2*J4 + 37507050*J8^2*J2^2 - 
        555660000*J8*J6^2 - 94382820*J8*J6*J4*J2 + 1049580*J8*J6*J2^3 +
        133698600*J8*J4^3 - 793800*J8*J4^2*J2^2 - 23756544*J6^2*J4^2 + 
        3087000*J6^2*J4*J2^2 + 98280*J6*J4^3*J2 + 47460*J6*J4^2*J2^3 - 
        980*J6*J4*J2^5 + 3831624*J4^5 + 2010015*J4^4*J2^2 - 
        102312*J4^3*J2^4 + 1029*J4^2*J2^6,
    1543147200*J10*J8 - 793618560*J10*J4^2 + 34292160*J10*J4*J2^2 - 
        357210*J10*J2^4 - 144670050*J8^2*J2 + 1538248320*J8*J6*J4 - 
        36435420*J8*J6*J2^2 - 82668600*J8*J4^2*J2 + 1905120*J8*J4*J2^3 
        + 254016000*J6^3 - 10160640*J6^2*J4*J2 - 38808*J6^2*J2^3 - 
        50388480*J6*J4^3 + 6373080*J6*J4^2*J2^2 - 278460*J6*J4*J2^4 + 
        3136*J6*J2^6 - 11809800*J4^4*J2 + 643545*J4^3*J2^3 - 
        7497*J4^2*J2^5,
    2540160*J10*J6 - 317520*J10*J4*J2 + 4410*J10*J2^3 + 1786050*J8^2 + 
        238140*J8*J6*J2 - 884520*J8*J4^2 + 133056*J6^2*J4 + 
        47880*J6*J4^2*J2 - 1092*J6*J4*J2^3 - 17496*J4^4 - 
        2205*J4^3*J2^2 + 49*J4^2*J2^4,
    J9,
    1093705578000*J8^3 + 291654820800*J8^2*J6*J2 - 1104121821600*J8^2*J4^2
        + 6076142100*J8^2*J4*J2^2 - 1469076134400*J8*J6^2*J4 + 
        36726903360*J8*J6^2*J2^2 + 231472080000*J8*J6*J4^2*J2 - 
        6631246440*J8*J6*J4*J2^3 + 30005640*J8*J6*J2^5 + 
        267846264000*J8*J4^4 - 23803045560*J8*J4^3*J2^2 + 
        540101520*J8*J4^2*J2^4 - 3333960*J8*J4*J2^6 - 256048128000*J6^4 +
        42247941120*J6^3*J4*J2 - 405409536*J6^3*J2^3 + 
        8888527872*J6^2*J4^3 - 5893679232*J6^2*J4^2*J2^2 + 
        274718304*J6^2*J4*J2^4 - 3093174*J6^2*J2^6 - 
        9523422720*J6*J4^4*J2 + 1237946976*J6*J4^3*J2^3 - 
        60328800*J6*J4^2*J2^5 + 1037232*J6*J4*J2^7 - 5488*J6*J2^9 + 
        5509980288*J4^6 - 1031704128*J4^5*J2^2 + 58796766*J4^4*J2^4 - 
        1091475*J4^3*J2^6 + 6174*J4^2*J2^8,
    J7,
    J5,
    J3
];
fC4 := x*(x^2 - 1) * (x^4 + a4 * x^2 + a0);

"_ C4 Lemma."; "";
"           Direct implication       :", IsLemmaDirectTrue(C4Stratum, fC4);

"           Converse implication     :",
    CheckInDepth select IsLemmaConverseTrue(C4Stratum: Group := <4,1>) else "not checked";

EQs := [Numerator(R[2]) : R in Genus3Determinants(ShiodaInvariants(fC4)) |
               R[1] in {[1, 2, 3], [1, 2, 4], [1, 5, 7], [1, 3, 6], [1, 2, 9]}];

SC := Scheme(AffineSpace(Universe(EQs)), GroebnerBasis(EQs));
IC := IrreducibleComponents(SC);
"";

"           Determinant cancelations :";
for i := 1 to #IC do
    Ia := ideal<Universe(EQs) |
	GroebnerBasis(Equations(ReducedSubscheme(IC[i])))>; Qa :=

    quo<Universe(EQs) | Ia>;

    Fa := RingOfFractions(Qa);
    q8, q0, q7, q1, q6, q2, q5, q3, q4 := Explode([Fa.i : i in [1..Rank(Fa)]]);

    z := PolynomialRing(Fa).1;
    fZ := z*(z^2 - 1) * (z^4 + q4 * z^2 + q0);

    printf "              At component ";
    for eqs in GroebnerBasis( Equations(ReducedSubscheme(IC[i]))) do
	printf "%o = 0, ", eqs;
    end for;
    printf "automorphism group is contained in %o;\n",
	IdentifyGroup(GeometricAutomorphismGroupFromShiodaInvariants(ShiodaInvariants(fZ)));
end for;


/* D4
 *******/
 D4Stratum := [
     1/36450*J2^4*J4^2 - 1/1215*J2*J3^2*J4^2 - 1/810*J2^2*J4^3 - 12/1225*J4^4 + 
     1/90*J3*J4^2*J5 - 26/42525*J2^3*J4*J6 + 52/2835*J3^2*J4*J6 + 
     76/2835*J2*J4^2*J6 - 2/15*J5^2*J6 + 352/4725*J4*J6^2 + 1/5*J4*J5*J7 - 
     2/15*J3*J6*J7 - 1/30*J2*J7^2 - 52/105*J4^2*J8 + 2/15*J2*J6*J8 + J8^2 + 
     1/15*J3*J4*J9 + 1/30*J2*J5*J9 - 2/5*J7*J9 + 1/405*J2^3*J10 - 
     2/27*J3^2*J10 - 8/45*J2*J4*J10 + 64/45*J6*J10,
     -1/29160*J2^3*J3*J4^2 + 1/972*J3^3*J4^2 + 1/648*J2*J3*J4^3 - 
     1/2160*J2^2*J4^2*J5 + 1/1134*J3*J4^2*J6 - 1/2430*J2^3*J5*J6 + 
     1/81*J3^2*J5*J6 + 31/3780*J2*J4*J5*J6 + 4/135*J5*J6^2 + 
     1/1620*J2^3*J4*J7 - 1/54*J3^2*J4*J7 - 1/140*J2*J4^2*J7 + 
     1/180*J2^2*J6*J7 - 17/315*J4*J6*J7 + 1/12*J3*J7^2 - 1/6*J3*J6*J8 - 
     1/12*J2*J7*J8 - 1/9720*J2^4*J9 + 1/324*J2*J3^2*J9 + 1/540*J2^2*J4*J9 + 
     2/105*J4^2*J9 - 1/12*J3*J5*J9 - 8/135*J2*J6*J9 + J8*J9 + 1/12*J3*J4*J10 
     + 1/24*J2*J5*J10 - 1/2*J7*J10,
     1/972000*J2^7*J4 - 1/16200*J2^4*J3^2*J4 + 1/1080*J2*J3^4*J4 - 
     1/10800*J2^5*J4^2 + 1/360*J2^2*J3^2*J4^2 + 311/168000*J2^3*J4^3 + 
     39/5600*J3^2*J4^3 - 153/2800*J2*J4^4 + 1/1200*J2^3*J3*J4*J5 - 
     1/40*J3^3*J4*J5 - 3/80*J2*J3*J4^2*J5 + 9/1600*J2^2*J4*J5^2 + 
     729/5600*J4^2*J5^2 + 13/324000*J2^6*J6 - 13/5400*J2^3*J3^2*J6 + 
     13/360*J3^4*J6 - 3247/756000*J2^4*J4*J6 + 3247/25200*J2*J3^2*J4*J6 + 
     941/8400*J2^2*J4^2*J6 - 9/50*J4^3*J6 - 1551/5600*J3*J4*J5*J6 - 
     351/1600*J2*J5^2*J6 + 1/288*J2^3*J6^2 - 5/48*J3^2*J6^2 + 
     211/840*J2*J4*J6^2 + J6^3 + 513/800*J3*J4^2*J7 + 3/800*J2^3*J5*J7 - 
     9/80*J3^2*J5*J7 + 81/1600*J2*J4*J5*J7 - 27/16*J5*J6*J7 + 
     13/800*J2^3*J4*J8 - 39/80*J3^2*J4*J8 - 549/400*J2*J4^2*J8 + 
     243/160*J5^2*J8 + 9/2*J4*J6*J8 - 1/400*J2^3*J3*J9 + 3/40*J3^3*J9 + 
     9/80*J2*J3*J4*J9 - 27/800*J2^2*J5*J9 + 81/100*J4*J5*J9 - 9/4*J3*J6*J9 + 
     1/400*J2^4*J10 - 3/40*J2*J3^2*J10 - 9/80*J2^2*J4*J10 - 81/25*J4^2*J10 + 
     81/80*J3*J5*J10 + 9/4*J2*J6*J10,
     -11/118098000*J2^7*J4 + 11/1968300*J2^4*J3^2*J4 - 11/131220*J2*J3^4*J4 + 
     19/2624400*J2^5*J4^2 - 19/87480*J2^2*J3^2*J4^2 - 23/637875*J2^3*J4^3 - 
     43/14175*J3^2*J4^3 - 1/3780*J2*J4^4 - 11/145800*J2^3*J3*J4*J5 + 
     11/4860*J3^3*J4*J5 + 19/6480*J2*J3*J4^2*J5 - 11/21600*J2^2*J4*J5^2 + 
     1/1400*J4^2*J5^2 - 7/4374000*J2^6*J6 + 7/72900*J2^3*J3^2*J6 - 
     7/4860*J3^4*J6 + 15137/91854000*J2^4*J4*J6 - 15137/3061800*J2*J3^2*J4*J6
     - 4231/1020600*J2^2*J4^2*J6 - 86/33075*J4^3*J6 - 433/226800*J3*J4*J5*J6 
     + 103/7200*J2*J5^2*J6 + 1/24300*J2^3*J6^2 - 1/810*J3^2*J6^2 - 
     511/18225*J2*J4*J6^2 - 103/25200*J3*J4^2*J7 - 11/32400*J2^3*J5*J7 + 
     11/1080*J3^2*J5*J7 + 3/800*J2*J4*J5*J7 + 1/180*J2*J3*J6*J7 + 
     1/360*J5*J6*J7 + 1/360*J2^2*J7^2 - 1/40*J4*J7^2 - 23/97200*J2^3*J4*J8 + 
     23/3240*J3^2*J4*J8 + 191/5400*J2*J4^2*J8 - 11/80*J5^2*J8 - 
     1/180*J2^2*J6*J8 + 61/189*J4*J6*J8 - 1/12*J3*J7*J8 + 1/8100*J2^3*J3*J9 -
     1/270*J3^3*J9 - 1/120*J2*J3*J4*J9 + 1/3600*J2^2*J5*J9 + 2/75*J4*J5*J9 + 
     2/45*J3*J6*J9 - 1/60*J2*J7*J9 - 11/48600*J2^4*J10 + 11/1620*J2*J3^2*J10 
     + 7/540*J2^2*J4*J10 - 46/525*J4^2*J10 - 1/20*J3*J5*J10 - 
     14/135*J2*J6*J10 + J8*J10,
     1/10497600*J2^7*J4 - 1/174960*J2^4*J3^2*J4 + 1/11664*J2*J3^4*J4 - 
     1/139968*J2^5*J4^2 + 5/23328*J2^2*J3^2*J4^2 - 37/907200*J2^3*J4^3 + 
     461/90720*J3^2*J4^3 - 13/15120*J2*J4^4 + 1/12960*J2^3*J3*J4*J5 - 
     1/432*J3^3*J4*J5 - 5/1728*J2*J3*J4^2*J5 + 1/1920*J2^2*J4*J5^2 + 
     31/1120*J4^2*J5^2 + 13/3499200*J2^6*J6 - 13/58320*J2^3*J3^2*J6 + 
     13/3888*J3^4*J6 - 3107/8164800*J2^4*J4*J6 + 3107/272160*J2*J3^2*J4*J6 + 
     2603/272160*J2^2*J4^2*J6 + 1/1890*J4^3*J6 - 13/2240*J3*J4*J5*J6 - 
     157/5760*J2*J5^2*J6 + 31/38880*J2^3*J6^2 - 31/1296*J3^2*J6^2 + 
     61/3780*J2*J4*J6^2 + 397/20160*J3*J4^2*J7 + 1/2880*J2^3*J5*J7 - 
     1/96*J3^2*J5*J7 + 29/1920*J2*J4*J5*J7 - 1/144*J2*J3*J6*J7 - 
     1/96*J5*J6*J7 - 1/288*J2^2*J7^2 - 1/32*J4*J7^2 + 13/8640*J2^3*J4*J8 - 
     13/288*J3^2*J4*J8 - 163/1440*J2*J4^2*J8 + 9/64*J5^2*J8 + 
     1/144*J2^2*J6*J8 + 1/12*J4*J6*J8 + 5/48*J3*J7*J8 - 1/9720*J2^3*J3*J9 + 
     1/324*J3^3*J9 + 7/864*J2*J3*J4*J9 + 1/2880*J2^2*J5*J9 - 3/10*J4*J5*J9 - 
     5/54*J3*J6*J9 - 1/48*J2*J7*J9 + J9^2 + 1/4320*J2^4*J10 - 
     1/144*J2*J3^2*J10 - 1/72*J2^2*J4*J10 - 2/15*J4^2*J10 + 1/24*J3*J5*J10 + 
     1/6*J2*J6*J10,
     -1/492075*J2^6*J3*J4 + 4/32805*J2^3*J3^3*J4 - 4/2187*J3^5*J4 + 
     2/10935*J2^4*J3*J4^2 - 4/729*J2*J3^3*J4^2 - 1/243*J2^2*J3*J4^3 + 
     6/35*J3*J4^4 - 1/18225*J2^5*J4*J5 + 2/1215*J2^2*J3^2*J4*J5 + 
     509/85050*J2^3*J4^2*J5 - 299/2835*J3^2*J4^2*J5 - 104/675*J2*J4^3*J5 - 
     1/90*J2*J3*J4*J5^2 + 3/10*J4*J5^3 + 937/382725*J2^3*J3*J4*J6 - 
     1874/25515*J3^3*J4*J6 - 937/8505*J2*J3*J4^2*J6 + 59/18225*J2^4*J5*J6 - 
     118/1215*J2*J3^2*J5*J6 - 3193/28350*J2^2*J4*J5*J6 - 1892/4725*J4^2*J5*J6
     + 59/45*J3*J5^2*J6 - 9272/8505*J3*J4*J6^2 + 20/81*J2*J5*J6^2 + 
     1/54675*J2^6*J7 - 4/3645*J2^3*J3^2*J7 + 4/243*J3^4*J7 - 
     119/36450*J2^4*J4*J7 + 119/1215*J2*J3^2*J4*J7 + 77/2025*J2^2*J4^2*J7 + 
     16/175*J4^3*J7 - 23/15*J3*J4*J5*J7 - 34/1215*J2^3*J6*J7 + 
     68/81*J3^2*J6*J7 + 1384/2835*J2*J4*J6*J7 + J5*J7^2 + 97/45*J3*J4^2*J8 + 
     1/135*J2^3*J5*J8 - 2/9*J3^2*J5*J8 + 49/90*J2*J4*J5*J8 - 6*J5*J6*J8 - 
     8/3*J4*J7*J8 + 1/6075*J2^5*J9 - 2/405*J2^2*J3^2*J9 - 8/675*J2^3*J4*J9 + 
     2/15*J3^2*J4*J9 + 148/225*J2*J4^2*J9 + 1/15*J2*J3*J5*J9 - 14/5*J5^2*J9 +
     32/135*J2^2*J6*J9 + 32/15*J4*J6*J9 + 4/3*J3*J7*J9 - 2/405*J2^3*J3*J10 + 
     4/27*J3^3*J10 + 2/9*J2*J3*J4*J10 - 1/15*J2^2*J5*J10 + 64/15*J4*J5*J10 - 
     64/9*J3*J6*J10 - 4/3*J2*J7*J10,
     -1/182250*J2^6*J3*J4 + 2/6075*J2^3*J3^3*J4 - 2/405*J3^5*J4 + 
     1/2025*J2^4*J3*J4^2 - 2/135*J2*J3^3*J4^2 - 1/90*J2^2*J3*J4^3 + 
     243/700*J3*J4^4 - 1/6750*J2^5*J4*J5 + 1/225*J2^2*J3^2*J4*J5 + 
     79/5250*J2^3*J4^2*J5 - 44/175*J3^2*J4^2*J5 - 2757/7000*J2*J4^3*J5 - 
     3/100*J2*J3*J4*J5^2 + 81/100*J4*J5^3 + 517/141750*J2^3*J3*J4*J6 - 
     517/4725*J3^3*J4*J6 - 517/3150*J2*J3*J4^2*J6 + 13/2250*J2^4*J5*J6 - 
     13/75*J2*J3^2*J5*J6 - 2213/10500*J2^2*J4*J5*J6 - 1086/875*J4^2*J5*J6 + 
     117/50*J3*J5^2*J6 - 1369/630*J3*J4*J6^2 + 1/4*J2*J5*J6^2 + 
     1/20250*J2^6*J7 - 2/675*J2^3*J3^2*J7 + 2/45*J3^4*J7 - 11/1500*J2^4*J4*J7
     + 11/50*J2*J3^2*J4*J7 + 29/250*J2^2*J4^2*J7 + 81/250*J4^3*J7 - 
     117/50*J3*J4*J5*J7 - 2/45*J2^3*J6*J7 + 4/3*J3^2*J6*J7 + 6/5*J2*J4*J6*J7 
     + J6^2*J7 + 171/50*J3*J4^2*J8 + 1/50*J2^3*J5*J8 - 3/5*J3^2*J5*J8 + 
     27/100*J2*J4*J5*J8 - 9*J5*J6*J8 + 1/2250*J2^5*J9 - 1/75*J2^2*J3^2*J9 - 
     29/750*J2^3*J4*J9 + 14/25*J3^2*J4*J9 + 177/125*J2*J4^2*J9 + 
     9/50*J2*J3*J5*J9 - 243/50*J5^2*J9 + 2/5*J2^2*J6*J9 + 24/5*J4*J6*J9 - 
     1/75*J2^3*J3*J10 + 2/5*J3^3*J10 + 3/5*J2*J3*J4*J10 - 9/50*J2^2*J5*J10 + 
     108/25*J4*J5*J10 - 12*J3*J6*J10,
     -1/7873200*J2^6*J3*J4 + 1/131220*J2^3*J3^3*J4 - 1/8748*J3^5*J4 + 
     1/77760*J2^4*J3*J4^2 - 1/2592*J2*J3^3*J4^2 - 5/15552*J2^2*J3*J4^3 + 
     1/280*J3*J4^4 - 1/291600*J2^5*J4*J5 + 1/9720*J2^2*J3^2*J4*J5 + 
     527/1814400*J2^3*J4^2*J5 - 53/15120*J3^2*J4^2*J5 - 1/200*J2*J4^3*J5 - 
     1/1440*J2*J3*J4*J5^2 + 3/160*J4*J5^3 + 307/6123600*J2^3*J3*J4*J6 - 
     307/204120*J3^3*J4*J6 - 13/5670*J2*J3*J4^2*J6 + 1/12150*J2^4*J5*J6 - 
     1/405*J2*J3^2*J5*J6 - 589/226800*J2^2*J4*J5*J6 - 33/1400*J4^2*J5*J6 + 
     19/720*J3*J5^2*J6 - 11/486*J3*J4*J6^2 - 17/2160*J2*J5*J6^2 + 
     1/874800*J2^6*J7 - 1/14580*J2^3*J3^2*J7 + 1/972*J3^4*J7 - 
     7/48600*J2^4*J4*J7 + 7/1620*J2*J3^2*J4*J7 + 241/151200*J2^2*J4^2*J7 + 
     3/1400*J4^3*J7 - 1/80*J3*J4*J5*J7 - 23/38880*J2^3*J6*J7 + 
     7/648*J3^2*J6*J7 + 97/5040*J2*J4*J6*J7 - 1/288*J2*J3*J7^2 + 
     37/720*J3*J4^2*J8 + 1/2160*J2^3*J5*J8 - 1/72*J3^2*J5*J8 - 
     7/480*J2*J4*J5*J8 + 1/144*J2*J3*J6*J8 + 1/48*J5*J6*J8 + 1/288*J2^2*J7*J8
     - 1/16*J4*J7*J8 + 17/1166400*J2^5*J9 - 17/38880*J2^2*J3^2*J9 - 
     7/7200*J2^3*J4*J9 + 7/540*J3^2*J4*J9 + 13/600*J2*J4^2*J9 + 
     11/1440*J2*J3*J5*J9 - 9/80*J5^2*J9 + 29/3240*J2^2*J6*J9 + 
     41/315*J4*J6*J9 - 1/12*J3*J7*J9 - 1/3240*J2^3*J3*J10 + 1/108*J3^3*J10 + 
     1/96*J2*J3*J4*J10 - 17/2880*J2^2*J5*J10 - 3/20*J4*J5*J10 - 
     7/36*J3*J6*J10 + 1/16*J2*J7*J10 + J9*J10,
     -1/295245000*J2^10 + 1/3280500*J2^7*J3^2 - 1/109350*J2^4*J3^4 + 
     1/10935*J2*J3^6 + 1/2187000*J2^8*J4 - 1/36450*J2^5*J3^2*J4 + 
     1/2430*J2^2*J3^4*J4 - 257/30618000*J2^6*J4^2 - 29/255150*J2^3*J3^2*J4^2 
     + 373/34020*J3^4*J4^2 - 353/486000*J2^4*J4^3 + 503/16200*J2*J3^2*J4^3 + 
     23/1050*J2^2*J4^4 + 18/625*J4^5 - 1/243000*J2^6*J3*J5 + 
     1/4050*J2^3*J3^3*J5 - 1/270*J3^5*J5 + 1/2700*J2^4*J3*J4*J5 - 
     1/90*J2*J3^3*J4*J5 - 1/120*J2^2*J3*J4^2*J5 + 209/8400*J3*J4^3*J5 - 
     1/18000*J2^5*J5^2 + 1/600*J2^2*J3^2*J5^2 + 1/240*J2^3*J4*J5^2 - 
     1/20*J3^2*J4*J5^2 - 793/5600*J2*J4^2*J5^2 - 3/400*J2*J3*J5^3 + 
     81/400*J5^4 - 1/1049760*J2^7*J6 + 1/17496*J2^4*J3^2*J6 - 
     5/5832*J2*J3^4*J6 + 1/11664*J2^5*J4*J6 - 5/1944*J2^2*J3^2*J4*J6 - 
     349/85050*J2^3*J4^2*J6 + 2959/45360*J3^2*J4^2*J6 + 103/1350*J2*J4^3*J6 -
     1/1296*J2^3*J3*J5*J6 + 5/216*J3^3*J5*J6 + 5/144*J2*J3*J4*J5*J6 - 
     1/192*J2^2*J5^2*J6 - 41/140*J4*J5^2*J6 - 13/14580*J2^4*J6^2 + 
     13/486*J2*J3^2*J6^2 + 13/324*J2^2*J4*J6^2 + 88/945*J4^2*J6^2 - 
     13/36*J3*J5*J6^2 + 19/6480*J2^3*J3*J4*J7 - 19/216*J3^3*J4*J7 - 
     19/144*J2*J3*J4^2*J7 + 1/720*J2^4*J5*J7 - 1/24*J2*J3^2*J5*J7 - 
     11/480*J2^2*J4*J5*J7 - 3/8*J4^2*J5*J7 + 9/16*J3*J5^2*J7 + 
     19/72*J3*J4*J6*J7 + 1/16*J2*J5*J6*J7 + 1/19440*J2^6*J8 - 
     1/324*J2^3*J3^2*J8 + 5/108*J3^4*J8 - 49/6480*J2^4*J4*J8 + 
     49/216*J2*J3^2*J4*J8 + 17/72*J2^2*J4^2*J8 + 7/5*J4^3*J8 - 
     19/16*J3*J4*J5*J8 - 9/32*J2*J5^2*J8 + 1/216*J2^3*J6*J8 - 5/36*J3^2*J6*J8
     - 17/36*J2*J4*J6*J8 - 7/10*J3*J4^2*J9 - 1/180*J2^3*J5*J9 + 
     1/6*J3^2*J5*J9 + 1/20*J2*J4*J5*J9 + J5*J6*J9 - 2/135*J2^3*J4*J10 + 
     4/9*J3^2*J4*J10 + 41/30*J2*J4^2*J10 - 9/4*J5^2*J10 - 4/3*J4*J6*J10,
     11/3321506250*J2^10 - 11/36905625*J2^7*J3^2 + 22/2460375*J2^4*J3^4 - 
     44/492075*J2*J3^6 - 61/147622500*J2^8*J4 + 61/2460375*J2^5*J3^2*J4 - 
     61/164025*J2^2*J3^4*J4 + 62/9568125*J2^6*J4^2 + 
     236/1913625*J2^3*J3^2*J4^2 - 1216/127575*J3^4*J4^2 + 
     3733/5467500*J2^4*J4^3 - 2504/91125*J2*J3^2*J4^3 - 9623/425250*J2^2*J4^4
     - 88/3125*J4^5 + 11/2733750*J2^6*J3*J5 - 22/91125*J2^3*J3^3*J5 + 
     22/6075*J3^5*J5 - 61/182250*J2^4*J3*J4*J5 + 61/6075*J2*J3^3*J4*J5 + 
     14/2025*J2^2*J3*J4^2*J5 - 1619/94500*J3*J4^3*J5 + 11/202500*J2^5*J5^2 - 
     11/6750*J2^2*J3^2*J5^2 - 7/1800*J2^3*J4*J5^2 + 11/225*J3^2*J4*J5^2 + 
     8713/63000*J2*J4^2*J5^2 + 11/1500*J2*J3*J5^3 - 99/500*J5^4 + 
     181/59049000*J2^7*J6 - 181/984150*J2^4*J3^2*J6 + 181/65610*J2*J3^4*J6 - 
     202/637875*J2^5*J4*J6 + 404/42525*J2^2*J3^2*J4*J6 + 
     4111/425250*J2^3*J4^2*J6 - 8287/170100*J3^2*J4^2*J6 - 
     11182/212625*J2*J4^3*J6 + 7/8100*J2^3*J3*J5*J6 - 7/270*J3^3*J5*J6 - 
     1571/28350*J2*J3*J4*J5*J6 - 11/2160*J2^2*J5^2*J6 + 158/1575*J4*J5^2*J6 +
     134/164025*J2^4*J6^2 - 268/10935*J2*J3^2*J6^2 - 2372/127575*J2^2*J4*J6^2
     - 32/405*J4^2*J6^2 + 28/135*J3*J5*J6^2 - 71/24300*J2^3*J3*J4*J7 + 
     71/810*J3^3*J4*J7 + 113/675*J2*J3*J4^2*J7 - 1/810*J2^4*J5*J7 + 
     1/27*J2*J3^2*J5*J7 + 83/2700*J2^2*J4*J5*J7 + 587/1050*J4^2*J5*J7 - 
     11/20*J3*J5^2*J7 - 11/1890*J3*J4*J6*J7 - 3/20*J2*J5*J6*J7 + 
     1/270*J2^3*J7^2 - 1/9*J3^2*J7^2 - 11/90*J2*J4*J7^2 - 11/218700*J2^6*J8 +
     11/3645*J2^3*J3^2*J8 - 11/243*J3^4*J8 + 301/36450*J2^4*J4*J8 - 
     301/1215*J2*J3^2*J4*J8 - 619/2025*J2^2*J4^2*J8 - 364/225*J4^3*J8 + 
     193/180*J3*J4*J5*J8 + 13/40*J2*J5^2*J8 - 11/2430*J2^3*J6*J8 + 
     11/81*J3^2*J6*J8 + 241/405*J2*J4*J6*J8 + J5*J7*J8 - 1/12150*J2^4*J3*J9 +
     1/405*J2*J3^3*J9 + 1/270*J2^2*J3*J4*J9 + 8/15*J3*J4^2*J9 + 
     7/2700*J2^3*J5*J9 - 1/9*J3^2*J5*J9 + 16/225*J2*J4*J5*J9 - 
     16/135*J2*J3*J6*J9 - 1/45*J2^2*J7*J9 - 16/15*J4*J7*J9 + 1/12150*J2^5*J10
     - 1/405*J2^2*J3^2*J10 + 17/1215*J2^3*J4*J10 - 43/81*J3^2*J4*J10 - 
     1052/675*J2*J4^2*J10 + 1/30*J2*J3*J5*J10 + 2*J5^2*J10 + 
     16/135*J2^2*J6*J10 + 224/135*J4*J6*J10 + 2/3*J3*J7*J10,
     1/246037500*J2^10 - 1/2733750*J2^7*J3^2 + 1/91125*J2^4*J3^4 - 
     2/18225*J2*J3^6 - 23/65610000*J2^8*J4 + 23/1093500*J2^5*J3^2*J4 - 
     23/72900*J2^2*J3^4*J4 - 17/2551500*J2^6*J4^2 + 103/170100*J2^3*J3^2*J4^2
     - 23/1890*J3^4*J4^2 + 11819/11340000*J2^4*J4^3 - 
     3823/126000*J2*J3^2*J4^3 - 23/750*J2^2*J4^4 - 81/21875*J4^5 + 
     1/202500*J2^6*J3*J5 - 1/3375*J2^3*J3^3*J5 + 1/225*J3^5*J5 - 
     23/81000*J2^4*J3*J4*J5 + 23/2700*J2*J3^3*J4*J5 + 1/360*J2^2*J3*J4^2*J5 -
     57/875*J3*J4^3*J5 + 1/15000*J2^5*J5^2 - 1/500*J2^2*J3^2*J5^2 - 
     47/12000*J2^3*J4*J5^2 + 3/50*J3^2*J4*J5^2 + 189/1000*J2*J4^2*J5^2 + 
     9/1000*J2*J3*J5^3 - 243/1000*J5^4 + 89/10935000*J2^7*J6 - 
     89/182250*J2^4*J3^2*J6 + 89/12150*J2*J3^4*J6 - 44101/51030000*J2^5*J4*J6
     + 44101/1701000*J2^2*J3^2*J4*J6 + 76333/2835000*J2^3*J4^2*J6 - 
     8537/63000*J3^2*J4^2*J6 - 464/2625*J2*J4^3*J6 + 1/3000*J2^3*J3*J5*J6 - 
     1/100*J3^3*J5*J6 - 8611/126000*J2*J3*J4*J5*J6 - 1/25*J2^2*J5^2*J6 + 
     963/1750*J4*J5^2*J6 + 149/97200*J2^4*J6^2 - 149/3240*J2*J3^2*J6^2 + 
     269/28350*J2^2*J4*J6^2 - 26/75*J4^2*J6^2 + 7/20*J3*J5*J6^2 - 
     29/9000*J2^3*J3*J4*J7 + 29/300*J3^3*J4*J7 + 537/2000*J2*J3*J4^2*J7 - 
     17/18000*J2^4*J5*J7 + 17/600*J2*J3^2*J5*J7 + 33/800*J2^2*J4*J5*J7 + 
     27/500*J4^2*J5*J7 - 27/40*J3*J5^2*J7 + 1/4*J3*J4*J6*J7 - 1/4*J2*J5*J6*J7
     - 1/16200*J2^6*J8 + 1/270*J2^3*J3^2*J8 - 1/18*J3^4*J8 + 
     643/54000*J2^4*J4*J8 - 643/1800*J2*J3^2*J4*J8 - 1603/3000*J2^2*J4^2*J8 -
     99/250*J4^3*J8 + 261/200*J3*J4*J5*J8 + 63/100*J2*J5^2*J8 + 
     1/180*J2^3*J6*J8 - 1/6*J3^2*J6*J8 + 11/30*J2*J4*J6*J8 + J6^2*J8 - 
     13/27000*J2^4*J3*J9 + 13/900*J2*J3^3*J9 + 13/600*J2^2*J3*J4*J9 + 
     72/125*J3*J4^2*J9 - 1/2000*J2^3*J5*J9 - 9/50*J3^2*J5*J9 + 
     3/250*J2*J4*J5*J9 - 13/30*J2*J3*J6*J9 + 13/27000*J2^5*J10 - 
     13/900*J2^2*J3^2*J10 - 37/3000*J2^3*J4*J10 - 7/25*J3^2*J4*J10 - 
     81/50*J2*J4^2*J10 + 39/200*J2*J3*J5*J10 + 243/100*J5^2*J10 + 
     13/30*J2^2*J6*J10 - 12/5*J4*J6*J10,
     11/738112500*J2^10 - 11/8201250*J2^7*J3^2 + 11/273375*J2^4*J3^4 - 
     22/54675*J2*J3^6 - 13/10935000*J2^8*J4 + 13/182250*J2^5*J3^2*J4 - 
     13/12150*J2^2*J3^4*J4 - 358/9568125*J2^6*J4^2 + 
     1747/637875*J2^3*J3^2*J4^2 - 2062/42525*J3^4*J4^2 + 
     36853/8505000*J2^4*J4^3 - 8557/70875*J2*J3^2*J4^3 - 423/3500*J2^2*J4^4 -
     972/21875*J4^5 + 11/607500*J2^6*J3*J5 - 11/10125*J2^3*J3^3*J5 + 
     11/675*J3^5*J5 - 13/13500*J2^4*J3*J4*J5 + 13/450*J2*J3^3*J4*J5 + 
     1/150*J2^2*J3*J4^2*J5 - 989/4200*J3*J4^3*J5 + 11/45000*J2^5*J5^2 - 
     11/1500*J2^2*J3^2*J5^2 - 83/6000*J2^3*J4*J5^2 + 11/50*J3^2*J4*J5^2 + 
     10027/14000*J2*J4^2*J5^2 + 33/1000*J2*J3*J5^3 - 891/1000*J5^4 + 
     2093/65610000*J2^7*J6 - 2093/1093500*J2^4*J3^2*J6 + 
     2093/72900*J2*J3^4*J6 - 42557/12757500*J2^5*J4*J6 + 
     42557/425250*J2^2*J3^2*J4*J6 + 420157/4252500*J2^3*J4^2*J6 - 
     226033/567000*J3^2*J4^2*J6 - 11897/23625*J2*J4^3*J6 + 
     187/81000*J2^3*J3*J5*J6 - 187/2700*J3^3*J5*J6 - 
     4601/15750*J2*J3*J4*J5*J6 - 573/4000*J2^2*J5^2*J6 + 1209/875*J4*J5^2*J6 
     + 121/18225*J2^4*J6^2 - 242/1215*J2*J3^2*J6^2 + 82/14175*J2^2*J4*J6^2 - 
     944/945*J4^2*J6^2 + 86/45*J3*J5*J6^2 - 1001/81000*J2^3*J3*J4*J7 + 
     1001/2700*J3^3*J4*J7 + 4511/4500*J2*J3*J4^2*J7 - 7/2250*J2^4*J5*J7 + 
     7/75*J2*J3^2*J5*J7 + 173/1500*J2^2*J4*J5*J7 + 537/500*J4^2*J5*J7 - 
     99/40*J3*J5^2*J7 - 667/1260*J3*J4*J6*J7 - 41/40*J2*J5*J6*J7 - 
     1/180*J2^3*J7^2 + 1/6*J3^2*J7^2 + 1/20*J2*J4*J7^2 + J6*J7^2 - 
     11/48600*J2^6*J8 + 11/810*J2^3*J3^2*J8 - 11/54*J3^4*J8 + 
     901/20250*J2^4*J4*J8 - 901/675*J2*J3^2*J4*J8 - 2239/1125*J2^2*J4^2*J8 - 
     482/125*J4^3*J8 + 1101/200*J3*J4*J5*J8 + 981/400*J2*J5^2*J8 + 
     7/540*J2^3*J6*J8 - 7/18*J3^2*J6*J8 + 223/90*J2*J4*J6*J8 - 
     1/500*J2^4*J3*J9 + 3/50*J2*J3^3*J9 + 9/100*J2^2*J3*J4*J9 + 
     362/125*J3*J4^2*J9 - 1/1000*J2^3*J5*J9 - 39/50*J3^2*J5*J9 + 
     3/125*J2*J4*J5*J9 - 8/5*J2*J3*J6*J9 + 1/10*J2^2*J7*J9 - 12/5*J4*J7*J9 + 
     1/500*J2^5*J10 - 3/50*J2^2*J3^2*J10 - 397/6750*J2^3*J4*J10 - 
     421/450*J3^2*J4*J10 - 2378/375*J2*J4^2*J10 + 81/100*J2*J3*J5*J10 + 
     207/25*J5^2*J10 + 8/5*J2^2*J6*J10 + 16/15*J4*J6*J10 - 3*J3*J7*J10,
     -11/53144100000*J2^10 + 11/590490000*J2^7*J3^2 - 11/19683000*J2^4*J3^4 + 
     11/1968300*J2*J3^6 + 293/7085880000*J2^8*J4 - 293/118098000*J2^5*J3^2*J4
     + 293/7873200*J2^2*J3^4*J4 - 20809/11022480000*J2^6*J4^2 + 
     14983/367416000*J2^3*J3^2*J4^2 + 971/2041200*J3^4*J4^2 - 
     23419/2449440000*J2^4*J4^3 + 4847/3024000*J2*J3^2*J4^3 + 
     3047/4536000*J2^2*J4^4 - 24/21875*J4^5 - 11/43740000*J2^6*J3*J5 + 
     11/729000*J2^3*J3^3*J5 - 11/48600*J3^5*J5 + 293/8748000*J2^4*J3*J4*J5 - 
     293/291600*J2*J3^3*J4*J5 - 761/777600*J2^2*J3*J4^2*J5 + 
     1403/504000*J3*J4^3*J5 - 11/3240000*J2^5*J5^2 + 11/108000*J2^2*J3^2*J5^2
     + 17/51840*J2^3*J4*J5^2 - 11/3600*J3^2*J4*J5^2 - 529/84000*J2*J4^2*J5^2 
     - 11/24000*J2*J3*J5^3 + 99/8000*J5^4 + 43/188956800*J2^7*J6 - 
     43/3149280*J2^4*J3^2*J6 + 43/209952*J2*J3^4*J6 - 
     1513/68890500*J2^5*J4*J6 + 1513/2296350*J2^2*J3^2*J4*J6 + 
     3257/12247200*J2^3*J4^2*J6 + 1417/181440*J3^2*J4^2*J6 + 
     5927/567000*J2*J4^3*J6 - 13/388800*J2^3*J3*J5*J6 + 13/12960*J3^3*J5*J6 +
     227/170100*J2*J3*J4*J5*J6 - 37/19200*J2^2*J5^2*J6 - 379/8400*J4*J5^2*J6 
     - 361/20995200*J2^4*J6^2 + 361/699840*J2*J3^2*J6^2 + 
     11699/2449440*J2^2*J4*J6^2 + 521/19845*J4^2*J6^2 - 1/360*J3*J5*J6^2 + 
     67/388800*J2^3*J3*J4*J7 - 67/12960*J3^3*J4*J7 - 949/151200*J2*J3*J4^2*J7
     + 13/97200*J2^4*J5*J7 - 13/3240*J2*J3^2*J5*J7 - 47/14400*J2^2*J4*J5*J7 +
     19/800*J4^2*J5*J7 + 11/320*J3*J5^2*J7 - 1/4320*J2^2*J3*J6*J7 - 
     1607/30240*J3*J4*J6*J7 - 7/8640*J2*J5*J6*J7 - 1/1728*J2^3*J7^2 + 
     1/72*J3^2*J7^2 + 1/320*J2*J4*J7^2 + 11/3499200*J2^6*J8 - 
     11/58320*J2^3*J3^2*J8 + 11/3888*J3^4*J8 - 101/291600*J2^4*J4*J8 + 
     101/9720*J2*J3^2*J4*J8 + 223/32400*J2^2*J4^2*J8 - 17/300*J4^3*J8 - 
     9/320*J3*J4*J5*J8 + 1/384*J2*J5^2*J8 + 1/1944*J2^3*J6*J8 - 
     11/1296*J3^2*J6*J8 - 7/6480*J2*J4*J6*J8 + 1/288*J2*J3*J7*J8 - 
     11/388800*J2^4*J3*J9 + 11/12960*J2*J3^3*J9 + 1/720*J2^2*J3*J4*J9 - 
     1/1800*J3*J4^2*J9 - 1/2400*J2^3*J5*J9 + 1/360*J3^2*J5*J9 - 
     1/900*J2*J4*J5*J9 - 19/2160*J2*J3*J6*J9 + 11/1440*J2^2*J7*J9 - 
     1/20*J4*J7*J9 + 19/583200*J2^5*J10 - 19/19440*J2^2*J3^2*J10 - 
     337/129600*J2^3*J4*J10 + 11/360*J3^2*J4*J10 + 1/18*J2*J4^2*J10 + 
     11/960*J2*J3*J5*J10 - 9/40*J5^2*J10 + 73/6480*J2^2*J6*J10 + 
     46/105*J4*J6*J10 - 1/4*J3*J7*J10 + J10^2,
     1/7873200*J2^7*J3*J4 - 1/131220*J2^4*J3^3*J4 + 1/8748*J2*J3^5*J4 - 
     1/87480*J2^5*J3*J4^2 + 1/2916*J2^2*J3^3*J4^2 + 7/34992*J2^3*J3*J4^3 + 
     5/2916*J3^3*J4^3 - 277/34020*J2*J3*J4^4 + 1/437400*J2^6*J4*J5 - 
     1/29160*J2^3*J3^2*J4*J5 - 1/972*J3^4*J4*J5 - 55/163296*J2^4*J4^2*J5 + 
     149/27216*J2*J3^2*J4^2*J5 + 359/37800*J2^2*J4^3*J5 + 9/350*J4^4*J5 + 
     1/1440*J2^2*J3*J4*J5^2 - 67/2520*J3*J4^2*J5^2 - 1/80*J2*J4*J5^3 + 
     7/2624400*J2^6*J3*J6 - 7/43740*J2^3*J3^3*J6 + 7/2916*J3^5*J6 - 
     2407/6123600*J2^4*J3*J4*J6 + 2407/204120*J2*J3^3*J4*J6 + 
     209/17010*J2^2*J3*J4^2*J6 + 5/3402*J3*J4^3*J6 - 19/145800*J2^5*J5*J6 + 
     19/4860*J2^2*J3^2*J5*J6 + 9761/2041200*J2^3*J4*J5*J6 - 
     803/27216*J3^2*J4*J5*J6 - 121/9450*J2*J4^2*J5*J6 - 97/1440*J2*J3*J5^2*J6
     + 11/60*J5^3*J6 - 1/14580*J2^3*J3*J6^2 + 1/486*J3^3*J6^2 + 
     2423/34020*J2*J3*J4*J6^2 - 53/3240*J2^2*J5*J6^2 - 16/567*J4*J5*J6^2 - 
     1/1312200*J2^7*J7 + 1/21870*J2^4*J3^2*J7 - 1/1458*J2*J3^4*J7 + 
     13/97200*J2^5*J4*J7 - 13/3240*J2^2*J3^2*J4*J7 + 1/170100*J2^3*J4^2*J7 + 
     13/45360*J3^2*J4^2*J7 - 31/1050*J2*J4^3*J7 + 1/6480*J2^3*J3*J5*J7 - 
     1/216*J3^3*J5*J7 + 107/1440*J2*J3*J4*J5*J7 - 7/20*J4*J5^2*J7 + 
     17/14580*J2^4*J6*J7 - 17/486*J2*J3^2*J6*J7 + 23/5670*J2^2*J4*J6*J7 - 
     136/945*J4^2*J6*J7 + 7/72*J3*J5*J6*J7 + 19/72*J3*J4*J7^2 + 
     7/6480*J2^3*J3*J4*J8 - 7/216*J3^3*J4*J8 - 11/60*J2*J3*J4^2*J8 - 
     1/3240*J2^4*J5*J8 + 1/108*J2*J3^2*J5*J8 - 19/720*J2^2*J4*J5*J8 + 
     67/60*J4^2*J5*J8 + 1/16*J3*J5^2*J8 - 1/4*J3*J4*J6*J8 + 1/24*J2*J5*J6*J8 
     + 1/216*J2^3*J7*J8 - 5/36*J3^2*J7*J8 - 11/36*J2*J4*J7*J8 - 
     1/291600*J2^6*J9 - 1/9720*J2^3*J3^2*J9 + 1/162*J3^4*J9 + 
     1/16200*J2^4*J4*J9 + 13/1080*J2*J3^2*J4*J9 - 19/675*J2^2*J4^2*J9 - 
     4/225*J4^3*J9 - 1/240*J2^2*J3*J5*J9 - 37/180*J3*J4*J5*J9 + 
     3/40*J2*J5^2*J9 - 1/81*J2^3*J6*J9 - 2/27*J3^2*J6*J9 - 1/9*J2*J4*J6*J9 - 
     1/12*J2*J3*J7*J9 + J5*J7*J9 + 1/3240*J2^4*J3*J10 - 1/108*J2*J3^3*J10 - 
     1/72*J2^2*J3*J4*J10 + 1/9*J3*J4^2*J10 - 1/2160*J2^3*J5*J10 + 
     5/36*J3^2*J5*J10 + 11/90*J2*J4*J5*J10 + 4/9*J2*J3*J6*J10 - 2*J5*J6*J10 +
     1/12*J2^2*J7*J10 - 4/3*J4*J7*J10,
     1/1399680*J2^7*J3*J4 - 1/23328*J2^4*J3^3*J4 + 5/7776*J2*J3^5*J4 - 
     1/15552*J2^5*J3*J4^2 + 5/2592*J2^2*J3^3*J4^2 + 1451/1209600*J2^3*J3*J4^3
     + 299/40320*J3^3*J4^3 - 229/6720*J2*J3*J4^4 + 1/108000*J2^6*J4*J5 + 
     1/43200*J2^3*J3^2*J4*J5 - 13/1440*J3^4*J4*J5 - 251/252000*J2^4*J4^2*J5 +
     43/11200*J2*J3^2*J4^2*J5 + 279/11200*J2^2*J4^3*J5 + 81/3500*J4^4*J5 + 
     1/256*J2^2*J3*J4*J5^2 + 549/22400*J3*J4^2*J5^2 - 81/1600*J2*J4*J5^3 + 
     17/1296000*J2^6*J3*J6 - 17/21600*J2^3*J3^3*J6 + 17/1440*J3^5*J6 - 
     9011/5443200*J2^4*J3*J4*J6 + 9011/181440*J2*J3^3*J4*J6 + 
     2899/60480*J2^2*J3*J4^2*J6 + 1/840*J3*J4^3*J6 - 43/108000*J2^5*J5*J6 + 
     43/3600*J2^2*J3^2*J5*J6 + 1571/100800*J2^3*J4*J5*J6 - 
     47/384*J3^2*J4*J5*J6 - 3/700*J2*J4^2*J5*J6 - 1491/6400*J2*J3*J5^2*J6 + 
     81/400*J5^3*J6 + 1/1152*J2^3*J3*J6^2 - 5/192*J3^3*J6^2 + 
     1475/6048*J2*J3*J4*J6^2 - 1/48*J2^2*J5*J6^2 - 41/140*J4*J5*J6^2 - 
     1/324000*J2^7*J7 + 1/5400*J2^4*J3^2*J7 - 1/360*J2*J3^4*J7 + 
     103/216000*J2^5*J4*J7 - 103/7200*J2^2*J3^2*J4*J7 - 73/12000*J2^3*J4^2*J7
     + 549/3200*J3^2*J4^2*J7 - 3/100*J2*J4^3*J7 + 13/9600*J2^3*J3*J5*J7 - 
     13/320*J3^3*J5*J7 + 1101/6400*J2*J3*J4*J5*J7 - 81/400*J4*J5^2*J7 + 
     1/288*J2^4*J6*J7 - 5/48*J2*J3^2*J6*J7 - 17/240*J2^2*J4*J6*J7 - 
     3/64*J3*J5*J6*J7 + 17/3200*J2^3*J3*J4*J8 - 51/320*J3^3*J4*J8 - 
     219/320*J2*J3*J4^2*J8 - 1/800*J2^4*J5*J8 + 3/80*J2*J3^2*J5*J8 - 
     39/1600*J2^2*J4*J5*J8 + 297/400*J4^2*J5*J8 + 351/640*J3*J5^2*J8 + 
     9/16*J3*J4*J6*J8 + 9/32*J2*J5*J6*J8 - 1/40500*J2^6*J9 - 
     11/43200*J2^3*J3^2*J9 + 43/1440*J3^4*J9 + 29/18000*J2^4*J4*J9 + 
     143/4800*J2*J3^2*J4*J9 - 3/40*J2^2*J4^2*J9 - 9/250*J4^3*J9 - 
     3/128*J2^2*J3*J5*J9 - 27/200*J3*J4*J5*J9 + 243/800*J2*J5^2*J9 - 
     13/360*J2^3*J6*J9 - 23/48*J3^2*J6*J9 - 1/20*J2*J4*J6*J9 + J6^2*J9 + 
     1/576*J2^4*J3*J10 - 5/96*J2*J3^3*J10 - 5/64*J2^2*J3*J4*J10 - 
     27/40*J3*J4^2*J10 + 1/80*J2^3*J5*J10 + 21/64*J3^2*J5*J10 - 
     9/80*J2*J4*J5*J10 + 25/16*J2*J3*J6*J10 - 9/4*J5*J6*J10,
     -193/196830000*J2^7*J3*J4 + 193/3280500*J2^4*J3^3*J4 - 193/218700*J2*J3^5*J4
     + 193/2187000*J2^5*J3*J4^2 - 193/72900*J2^2*J3^3*J4^2 - 
     169/90000*J2^3*J3*J4^3 - 131/40500*J3^3*J4^3 + 1313/23625*J2*J3*J4^4 - 
     1/45000*J2^6*J4*J5 + 131/243000*J2^3*J3^2*J4*J5 + 31/8100*J3^4*J4*J5 + 
     25687/11340000*J2^4*J4^2*J5 - 451/14000*J2*J3^2*J4^2*J5 - 
     9269/157500*J2^2*J4^3*J5 + 27/8750*J4^4*J5 - 193/36000*J2^2*J3*J4*J5^2 -
     73/7000*J3*J4^2*J5^2 + 243/2000*J2*J4*J5^3 - 7/1458000*J2^6*J3*J6 + 
     7/24300*J2^3*J3^3*J6 - 7/1620*J3^5*J6 + 159631/153090000*J2^4*J3*J4*J6 -
     159631/5103000*J2*J3^3*J4*J6 - 31639/850500*J2^2*J3*J4^2*J6 - 
     131/47250*J3*J4^3*J6 + 349/405000*J2^5*J5*J6 - 349/13500*J2^2*J3^2*J5*J6
     - 61693/1890000*J2^3*J4*J5*J6 + 23899/378000*J3^2*J4*J5*J6 - 
     4279/39375*J2*J4^2*J5*J6 + 1501/4000*J2*J3*J5^2*J6 - 99/500*J5^3*J6 + 
     1/8100*J2^3*J3*J6^2 - 1/270*J3^3*J6^2 - 65179/170100*J2*J3*J4*J6^2 + 
     73/1800*J2^2*J5*J6^2 + 104/525*J4*J5*J6^2 + 1/135000*J2^7*J7 - 
     1/2250*J2^4*J3^2*J7 + 1/150*J2*J3^4*J7 - 889/810000*J2^5*J4*J7 + 
     889/27000*J2^2*J3^2*J4*J7 + 416/23625*J2^3*J4^2*J7 - 
     9377/126000*J3^2*J4^2*J7 + 96/4375*J2*J4^3*J7 - 31/54000*J2^3*J3*J5*J7 +
     31/1800*J3^3*J5*J7 - 4093/12000*J2*J3*J4*J5*J7 + 99/500*J4*J5^2*J7 - 
     1/150*J2^4*J6*J7 + 1/5*J2*J3^2*J6*J7 + 107/630*J2^2*J4*J6*J7 - 
     86/525*J4^2*J6*J7 - 17/120*J3*J5*J6*J7 + 1/8*J3*J4*J7^2 - 
     7/3600*J2^3*J3*J4*J8 + 7/120*J3^3*J4*J8 + 499/750*J2*J3*J4^2*J8 + 
     3/1000*J2^4*J5*J8 - 9/100*J2*J3^2*J5*J8 + 187/6000*J2^2*J4*J5*J8 - 
     213/500*J4^2*J5*J8 - 93/400*J3*J5^2*J8 + 3/20*J3*J4*J6*J8 - 
     41/40*J2*J5*J6*J8 + 1/360*J2^3*J7*J8 - 1/12*J3^2*J7*J8 - 
     3/20*J2*J4*J7*J8 + J6*J7*J8 + 163/2430000*J2^6*J9 - 
     133/81000*J2^3*J3^2*J9 - 1/90*J3^4*J9 - 461/81000*J2^4*J4*J9 + 
     343/5400*J2*J3^2*J4*J9 + 1183/5625*J2^2*J4^2*J9 - 28/625*J4^3*J9 + 
     193/6000*J2^2*J3*J5*J9 + 47/500*J3*J4*J5*J9 - 729/1000*J2*J5^2*J9 + 
     43/675*J2^3*J6*J9 + 2/15*J3^2*J6*J9 + 29/45*J2*J4*J6*J9 - 
     1/20*J2*J3*J7*J9 - 193/81000*J2^4*J3*J10 + 193/2700*J2*J3^3*J10 + 
     193/1800*J2^2*J3*J4*J10 + 14/125*J3*J4^2*J10 - 533/18000*J2^3*J5*J10 - 
     23/300*J3^2*J5*J10 + 68/125*J2*J4*J5*J10 - 92/45*J2*J3*J6*J10 + 
     2*J5*J6*J10 + 1/20*J2^2*J7*J10 - 6/5*J4*J7*J10,
     -241/49207500*J2^7*J3*J4 + 241/820125*J2^4*J3^3*J4 - 241/54675*J2*J3^5*J4 + 
     241/546750*J2^5*J3*J4^2 - 241/18225*J2^2*J3^3*J4^2 - 
     352361/38272500*J2^3*J3*J4^3 - 13607/637875*J3^3*J4^3 + 
     57713/212625*J2*J3*J4^4 - 91/911250*J2^6*J4*J5 + 
     41/20250*J2^3*J3^2*J4*J5 + 59/2025*J3^4*J4*J5 + 7411/729000*J2^4*J4^2*J5
     - 3073/24300*J2*J3^2*J4^2*J5 - 4587/17500*J2^2*J4^3*J5 + 28/625*J4^4*J5 
     - 241/9000*J2^2*J3*J4*J5^2 - 3701/31500*J3*J4^2*J5^2 + 
     273/500*J2*J4*J5^3 - 131/3280500*J2^6*J3*J6 + 131/54675*J2^3*J3^3*J6 - 
     131/3645*J3^5*J6 + 255847/38272500*J2^4*J3*J4*J6 - 
     255847/1275750*J2*J3^3*J4*J6 - 46768/212625*J2^2*J3*J4^2*J6 - 
     2152/212625*J3*J4^3*J6 + 1189/303750*J2^5*J5*J6 - 
     1189/10125*J2^2*J3^2*J5*J6 - 210857/1417500*J2^3*J4*J5*J6 + 
     40631/94500*J3^2*J4*J5*J6 - 22151/50625*J2*J4^2*J5*J6 + 
     5411/3000*J2*J3*J5^2*J6 - 253/375*J5^3*J6 - 16/18225*J2^3*J3*J6^2 + 
     32/1215*J3^3*J6^2 - 78982/42525*J2*J3*J4*J6^2 + 127/675*J2^2*J5*J6^2 + 
     16/21*J4*J5*J6^2 + 91/2733750*J2^7*J7 - 182/91125*J2^4*J3^2*J7 + 
     182/6075*J2*J3^4*J7 - 1003/202500*J2^5*J4*J7 + 1003/6750*J2^2*J3^2*J4*J7
     + 9188/118125*J2^3*J4^2*J7 - 18533/31500*J3^2*J4^2*J7 + 
     14369/118125*J2*J4^3*J7 - 59/13500*J2^3*J3*J5*J7 + 59/450*J3^3*J5*J7 - 
     4721/3000*J2*J3*J4*J5*J7 + 127/250*J4*J5^2*J7 - 184/6075*J2^4*J6*J7 + 
     368/405*J2*J3^2*J6*J7 + 7043/9450*J2^2*J4*J6*J7 - 344/525*J4^2*J6*J7 + 
     1/10*J3*J5*J6*J7 + 1/3*J3*J4*J7^2 + J7^3 - 131/8100*J2^3*J3*J4*J8 + 
     131/270*J3^3*J4*J8 + 4103/1125*J2*J3*J4^2*J8 + 91/6750*J2^4*J5*J8 - 
     91/225*J2*J3^2*J5*J8 + 229/1500*J2^2*J4*J5*J8 - 686/375*J4^2*J5*J8 - 
     177/100*J3*J5^2*J8 - 34/45*J3*J4*J6*J8 - 139/30*J2*J5*J6*J8 + 
     1/90*J2^3*J7*J8 - 1/3*J3^2*J7*J8 - 13/30*J2*J4*J7*J8 + 
     181/607500*J2^6*J9 - 121/20250*J2^3*J3^2*J9 - 4/45*J3^4*J9 - 
     418/16875*J2^4*J4*J9 + 467/2250*J2*J3^2*J4*J9 + 1747/1875*J2^2*J4^2*J9 -
     1792/5625*J4^3*J9 + 241/1500*J2^2*J3*J5*J9 + 48/125*J3*J4*J5*J9 - 
     819/250*J2*J5^2*J9 + 8/27*J2^3*J6*J9 + 64/45*J3^2*J6*J9 + 
     8/3*J2*J4*J6*J9 - 1/5*J2*J3*J7*J9 - 241/20250*J2^4*J3*J10 + 
     241/675*J2*J3^3*J10 + 241/450*J2^2*J3*J4*J10 + 214/125*J3*J4^2*J10 - 
     599/4500*J2^3*J5*J10 - 62/75*J3^2*J5*J10 + 291/125*J2*J4*J5*J10 - 
     464/45*J2*J3*J6*J10 + 32/5*J5*J6*J10 + 1/5*J2^2*J7*J10 - 4/5*J4*J7*J10,
     -7/8857350000*J2^11 + 7/98415000*J2^8*J3^2 - 7/3280500*J2^5*J3^4 + 
     7/328050*J2^2*J3^6 + 373/3542940000*J2^9*J4 - 17/2460375*J2^6*J3^2*J4 + 
     19/145800*J2^3*J3^4*J4 - 7/13122*J3^6*J4 - 73303/24800580000*J2^7*J4^2 +
     37393/413343000*J2^4*J3^2*J4^2 - 1483/27556200*J2*J3^4*J4^2 - 
     193469/2755620000*J2^5*J4^3 + 136897/45927000*J2^2*J3^2*J4^3 + 
     117671/30618000*J2^3*J4^4 + 14989/680400*J3^2*J4^4 - 
     49561/3543750*J2*J4^5 - 7/7290000*J2^7*J3*J5 + 7/121500*J2^4*J3^3*J5 - 
     7/8100*J2*J3^5*J5 + 101/1458000*J2^5*J3*J4*J5 - 
     101/48600*J2^2*J3^3*J4*J5 - 5123/6123600*J2^3*J3*J4^2*J5 - 
     2059/204120*J3^3*J4^2*J5 - 52049/6804000*J2*J3*J4^3*J5 - 
     7/540000*J2^6*J5^2 + 7/18000*J2^3*J3^2*J5^2 + 109/129600*J2^4*J4*J5^2 - 
     121/10800*J2*J3^2*J4*J5^2 - 35083/1134000*J2^2*J4^2*J5^2 + 
     17/420*J4^3*J5^2 - 7/4000*J2^2*J3*J5^3 + 3/80*J3*J4*J5^3 + 
     189/4000*J2*J5^4 - 7/10497600*J2^8*J6 + 7/174960*J2^5*J3^2*J6 - 
     7/11664*J2^2*J3^4*J6 + 1973/27556200*J2^6*J4*J6 - 
     3179/1837080*J2^3*J3^2*J4*J6 - 767/61236*J3^4*J4*J6 - 
     143047/55112400*J2^4*J4^2*J6 + 21523/7348320*J2*J3^2*J4^2*J6 + 
     52666/1913625*J2^2*J4^3*J6 - 4148/70875*J4^4*J6 + 11/23328*J2^4*J3*J5*J6
     - 55/3888*J2*J3^3*J5*J6 - 247/22680*J2^2*J3*J4*J5*J6 - 
     36703/340200*J3*J4^2*J5*J6 + 301/155520*J2^3*J5^2*J6 + 
     157/648*J3^2*J5^2*J6 - 341/3600*J2*J4*J5^2*J6 - 43/218700*J2^5*J6^2 + 
     43/7290*J2^2*J3^2*J6^2 + 4957/765450*J2^3*J4*J6^2 - 
     6877/25515*J3^2*J4*J6^2 - 638/54675*J2*J4^2*J6^2 - 1/405*J2*J3*J5*J6^2 +
     11/135*J5^2*J6^2 + 7/1312200*J2^6*J3*J7 - 7/21870*J2^3*J3^3*J7 + 
     7/1458*J3^5*J7 - 71/583200*J2^4*J3*J4*J7 + 71/19440*J2*J3^3*J4*J7 - 
     1/36*J2^2*J3*J4^2*J7 + 4457/37800*J3*J4^3*J7 + 11/32400*J2^5*J5*J7 - 
     11/1080*J2^2*J3^2*J5*J7 - 421/38880*J2^3*J4*J5*J7 - 
     851/3240*J3^2*J4*J5*J7 + 17/180*J2*J4^2*J5*J7 + 67/480*J2*J3*J5^2*J7 - 
     9/40*J5^3*J7 - 37/7290*J2^3*J3*J6*J7 + 37/243*J3^3*J6*J7 + 
     247/5040*J2*J3*J4*J6*J7 + 127/4320*J2^2*J5*J6*J7 + 373/3780*J4*J5*J6*J7 
     - 1/1296*J2^4*J7^2 + 5/216*J2*J3^2*J7^2 + 5/432*J2^2*J4*J7^2 - 
     1/15*J4^2*J7^2 + 11/1049760*J2^7*J8 - 11/17496*J2^4*J3^2*J8 + 
     55/5832*J2*J3^4*J8 - 527/291600*J2^5*J4*J8 + 527/9720*J2^2*J3^2*J4*J8 + 
     286/3645*J2^3*J4^2*J8 + 239/1944*J3^2*J4^2*J8 - 2533/8100*J2*J4^3*J8 + 
     1/3240*J2^3*J3*J5*J8 - 1/108*J3^3*J5*J8 - 43/480*J2*J3*J4*J5*J8 - 
     27/320*J2^2*J5^2*J8 + 73/120*J4*J5^2*J8 + 19/58320*J2^4*J6*J8 - 
     19/1944*J2*J3^2*J6*J8 - 247/3240*J2^2*J4*J6*J8 + 142/405*J4^2*J6*J8 - 
     41/36*J3*J5*J6*J8 - 5/36*J3*J4*J7*J8 + 1/19440*J2^5*J3*J9 - 
     1/648*J2^2*J3^3*J9 - 469/97200*J2^3*J3*J4*J9 + 61/810*J3^3*J4*J9 + 
     61/675*J2*J3*J4^2*J9 - 7/4320*J2^4*J5*J9 + 5/72*J2*J3^2*J5*J9 + 
     11/540*J2^2*J4*J5*J9 + 8/45*J4^2*J5*J9 - 17/20*J3*J5^2*J9 + 
     2/27*J2^2*J3*J6*J9 + 20/27*J3*J4*J6*J9 + 11/1080*J2^3*J7*J9 + 
     1/9*J3^2*J7*J9 - 1/9*J2*J4*J7*J9 - 53/874800*J2^6*J10 + 
     61/29160*J2^3*J3^2*J10 - 2/243*J3^4*J10 + 43/16200*J2^4*J4*J10 - 
     11/1080*J2*J3^2*J4*J10 + 38/225*J2^2*J4^2*J10 - 176/225*J4^3*J10 - 
     1/48*J2^2*J3*J5*J10 + 119/90*J3*J4*J5*J10 - 7/20*J2*J5^2*J10 - 
     46/1215*J2^3*J6*J10 - 88/81*J3^2*J6*J10 + 28/135*J2*J4*J6*J10 - 
     5/12*J2*J3*J7*J10 + J5*J7*J10,
     -11/11809800000*J2^11 + 11/131220000*J2^8*J3^2 - 11/4374000*J2^5*J3^4 + 
     11/437400*J2^2*J3^6 + 569/6298560000*J2^9*J4 - 139/20995200*J2^6*J3^2*J4
     + 1073/6998400*J2^3*J3^4*J4 - 7/6480*J3^6*J4 + 317/1224720000*J2^7*J4^2 
     + 499/27216000*J2^4*J3^2*J4^2 - 2131/2721600*J2*J3^4*J4^2 - 
     206413/1088640000*J2^5*J4^3 + 132563/36288000*J2^2*J3^2*J4^3 + 
     1305533/211680000*J2^3*J4^4 + 155179/2352000*J3^2*J4^4 - 
     16721/2450000*J2*J4^5 - 11/9720000*J2^7*J3*J5 + 11/162000*J2^4*J3^3*J5 -
     11/10800*J2*J3^5*J5 + 317/7776000*J2^5*J3*J4*J5 - 
     317/259200*J2^2*J3^3*J4*J5 + 3889/2016000*J2^3*J3*J4^2*J5 - 
     4451/100800*J3^3*J4^2*J5 - 18721/336000*J2*J3*J4^3*J5 - 
     11/720000*J2^6*J5^2 + 11/24000*J2^3*J3^2*J5^2 + 
     1001/1152000*J2^4*J4*J5^2 - 57/3200*J2*J3^2*J4*J5^2 - 
     9221/224000*J2^2*J4^2*J5^2 + 162/6125*J4^3*J5^2 - 33/16000*J2^2*J3*J5^3 
     + 459/3200*J3*J4*J5^3 + 891/16000*J2*J5^4 - 3457/2099520000*J2^8*J6 + 
     3457/34992000*J2^5*J3^2*J6 - 3457/2332800*J2^2*J3^4*J6 + 
     24431/139968000*J2^6*J4*J6 - 5909/1306368*J2^3*J3^2*J4*J6 - 
     647/30240*J3^4*J4*J6 - 1160029/211680000*J2^4*J4^2*J6 - 
     209539/63504000*J2*J3^2*J4^2*J6 + 196531/5292000*J2^2*J4^3*J6 - 
     299/8750*J4^4*J6 + 151/144000*J2^4*J3*J5*J6 - 151/4800*J2*J3^3*J5*J6 - 
     65041/2419200*J2^2*J3*J4*J5*J6 - 275189/1176000*J3*J4^2*J5*J6 + 
     593/76800*J2^3*J5^2*J6 + 741/1600*J3^2*J5^2*J6 - 
     2721/22400*J2*J4*J5^2*J6 - 3143/9331200*J2^5*J6^2 + 
     3143/311040*J2^2*J3^2*J6^2 + 149/217728*J2^3*J4*J6^2 - 
     133/288*J3^2*J4*J6^2 + 4573/66150*J2*J4^2*J6^2 - 1/30*J2*J3*J5*J6^2 + 
     7/648000*J2^6*J3*J7 - 7/10800*J2^3*J3^3*J7 + 7/720*J3^5*J7 - 
     43/54000*J2^4*J3*J4*J7 + 43/1800*J2*J3^3*J4*J7 - 
     423/12800*J2^2*J3*J4^2*J7 + 6099/56000*J3*J4^3*J7 + 
     461/1728000*J2^5*J5*J7 - 461/57600*J2^2*J3^2*J5*J7 - 
     1063/107520*J2^3*J4*J5*J7 - 5317/11200*J3^2*J4*J5*J7 + 
     747/22400*J2*J4^2*J5*J7 + 513/3200*J2*J3*J5^2*J7 - 243/1600*J5^3*J7 - 
     13/1440*J2^3*J3*J6*J7 + 13/48*J3^3*J6*J7 + 5/24*J2*J3*J4*J6*J7 + 
     187/3840*J2^2*J5*J6*J7 + 183/1120*J4*J5*J6*J7 + 17/1296000*J2^7*J8 - 
     17/21600*J2^4*J3^2*J8 + 17/1440*J2*J3^4*J8 - 13207/5184000*J2^5*J4*J8 + 
     13207/172800*J2^2*J3^2*J4*J8 + 635/5376*J2^3*J4^2*J8 + 
     3421/6720*J3^2*J4^2*J8 - 4637/28000*J2*J4^3*J8 + 1/320*J2^3*J3*J5*J8 - 
     3/32*J3^3*J5*J8 - 339/1600*J2*J3*J4*J5*J8 - 1803/12800*J2^2*J5^2*J8 + 
     297/448*J4*J5^2*J8 - 1/1440*J2^4*J6*J8 + 1/48*J2*J3^2*J6*J8 - 
     29/288*J2^2*J4*J6*J8 - 103/140*J4^2*J6*J8 - 27/16*J3*J5*J6*J8 + 
     19/103680*J2^5*J3*J9 - 19/3456*J2^2*J3^3*J9 - 
     26273/2016000*J2^3*J3*J4*J9 + 201/1400*J3^3*J4*J9 + 
     1059/5600*J2*J3*J4^2*J9 - 173/192000*J2^4*J5*J9 + 81/800*J2*J3^2*J5*J9 +
     571/28000*J2^2*J4*J5*J9 + 207/7000*J4^2*J5*J9 - 513/400*J3*J5^2*J9 + 
     95/576*J2^2*J3*J6*J9 + 369/280*J3*J4*J6*J9 - 323/2592000*J2^6*J10 + 
     19/9600*J2^3*J3^2*J10 + 19/360*J3^4*J10 + 11321/2016000*J2^4*J4*J10 + 
     221/2800*J2*J3^2*J4*J10 + 59/224*J2^2*J4^2*J10 - 198/875*J4^3*J10 - 
     19/256*J2^2*J3*J5*J10 + 7263/5600*J3*J4*J5*J10 - 81/200*J2*J5^2*J10 - 
     17/192*J2^3*J6*J10 - 55/24*J3^2*J6*J10 + 51/280*J2*J4*J6*J10 + J6^2*J10,
     -7/3936600000*J2^11 + 7/43740000*J2^8*J3^2 - 7/1458000*J2^5*J3^4 + 
     7/145800*J2^2*J3^6 + 97/524880000*J2^9*J4 - 17/1312200*J2^6*J3^2*J4 + 
     487/1749600*J2^3*J3^4*J4 - 49/29160*J3^6*J4 - 2069/3674160000*J2^7*J4^2 
     + 1649/61236000*J2^4*J3^2*J4^2 - 1229/4082400*J2*J3^4*J4^2 - 
     142103/408240000*J2^5*J4^3 + 13891/1701000*J2^2*J3^2*J4^3 + 
     260963/22680000*J2^3*J4^4 + 135239/1512000*J3^2*J4^4 - 
     30361/1575000*J2*J4^5 - 7/3240000*J2^7*J3*J5 + 7/54000*J2^4*J3^3*J5 - 
     7/3600*J2*J3^5*J5 + 193/1944000*J2^5*J3*J4*J5 - 
     193/64800*J2^2*J3^3*J4*J5 + 2861/1512000*J2^3*J3*J4^2*J5 - 
     3001/50400*J3^3*J4^2*J5 - 71383/1008000*J2*J3*J4^3*J5 - 
     7/240000*J2^6*J5^2 + 7/8000*J2^3*J3^2*J5^2 + 11/6400*J2^4*J4*J5^2 - 
     151/4800*J2*J3^2*J4*J5^2 - 5281/67200*J2^2*J4^2*J5^2 + 
     1119/14000*J4^3*J5^2 - 63/16000*J2^2*J3*J5^3 + 333/1600*J3*J4*J5^3 + 
     1701/16000*J2*J5^4 - 307/116640000*J2^8*J6 + 307/1944000*J2^5*J3^2*J6 - 
     307/129600*J2^2*J3^4*J6 + 56459/204120000*J2^6*J4*J6 - 
     4657/637875*J2^3*J3^2*J4*J6 - 20353/680400*J3^4*J4*J6 - 
     1811459/204120000*J2^4*J4^2*J6 + 213761/27216000*J2*J3^2*J4^2*J6 + 
     4789/70875*J2^2*J4^3*J6 - 3121/39375*J4^4*J6 + 
     1847/1296000*J2^4*J3*J5*J6 - 1847/43200*J2*J3^3*J5*J6 - 
     10459/302400*J2^2*J3*J4*J5*J6 - 31739/84000*J3*J4^2*J5*J6 + 
     3893/345600*J2^3*J5^2*J6 + 4831/7200*J3^2*J5^2*J6 - 
     11971/56000*J2*J4*J5^2*J6 - 61/97200*J2^5*J6^2 + 61/3240*J2^2*J3^2*J6^2 
     + 569/75600*J2^3*J4*J6^2 - 8327/11340*J3^2*J4*J6^2 + 
     1357/28350*J2*J4^2*J6^2 - 83/720*J2*J3*J5*J6^2 + 11/60*J5^2*J6^2 + 
     49/2916000*J2^6*J3*J7 - 49/48600*J2^3*J3^3*J7 + 49/3240*J3^5*J7 - 
     1151/1296000*J2^4*J3*J4*J7 + 1151/43200*J2*J3^3*J4*J7 - 
     101/1600*J2^2*J3*J4^2*J7 + 18769/84000*J3*J4^3*J7 + 1/1800*J2^5*J5*J7 - 
     1/60*J2^2*J3^2*J5*J7 - 103/6000*J2^3*J4*J5*J7 - 537/800*J3^2*J4*J5*J7 + 
     127/2000*J2*J4^2*J5*J7 + 981/3200*J2*J3*J5^2*J7 - 243/800*J5^3*J7 - 
     41/3240*J2^3*J3*J6*J7 + 41/108*J3^3*J6*J7 + 485/1344*J2*J3*J4*J6*J7 + 
     53/640*J2^2*J5*J6*J7 - 3/16*J4*J5*J6*J7 + 1/2880*J2^4*J7^2 - 
     1/96*J2*J3^2*J7^2 - 1/192*J2^2*J4*J7^2 - 3/40*J4^2*J7^2 + 
     97/3888000*J2^7*J8 - 97/64800*J2^4*J3^2*J8 + 97/4320*J2*J3^4*J8 - 
     251/54000*J2^5*J4*J8 + 251/1800*J2^2*J3^2*J4*J8 + 
     22483/108000*J2^3*J4^2*J8 + 4219/7200*J3^2*J4^2*J8 - 
     1931/6000*J2*J4^3*J8 + 31/7200*J2^3*J3*J5*J8 - 31/240*J3^3*J5*J8 - 
     1529/3200*J2*J3*J4*J5*J8 - 1611/6400*J2^2*J5^2*J8 + 963/800*J4*J5^2*J8 -
     7/8640*J2^4*J6*J8 + 7/288*J2*J3^2*J6*J8 - 97/480*J2^2*J4*J6*J8 + 
     13/30*J4^2*J6*J8 - 39/16*J3*J5*J6*J8 + 1/16*J3*J4*J7*J8 + 
     37/129600*J2^5*J3*J9 - 37/4320*J2^2*J3^3*J9 - 4507/216000*J2^3*J3*J4*J9 
     + 433/1800*J3^3*J4*J9 + 21/100*J2*J3*J4^2*J9 - 359/144000*J2^4*J5*J9 + 
     457/2400*J2*J3^2*J5*J9 + 323/6000*J2^2*J4*J5*J9 + 18/125*J4^2*J5*J9 - 
     837/400*J3*J5^2*J9 + 11/45*J2^2*J3*J6*J9 + 61/30*J3*J4*J6*J9 - 
     1/288*J2^3*J7*J9 - 1/12*J3^2*J7*J9 + J6*J7*J9 - 137/648000*J2^6*J10 + 
     89/21600*J2^3*J3^2*J10 + 1/15*J3^4*J10 + 341/36000*J2^4*J4*J10 + 
     81/800*J2*J3^2*J4*J10 + 293/600*J2^2*J4^2*J10 - 124/125*J4^3*J10 - 
     37/320*J2^2*J3*J5*J10 + 203/100*J3*J4*J5*J10 - 297/400*J2*J5^2*J10 - 
     13/90*J2^3*J6*J10 - 3*J3^2*J6*J10 + 2/5*J2*J4*J6*J10 + 
     3/16*J2*J3*J7*J10,
     91/33215062500*J2^11 - 91/369056250*J2^8*J3^2 + 91/12301875*J2^5*J3^4 - 
     182/2460375*J2^2*J3^6 - 44/184528125*J2^9*J4 + 568/36905625*J2^6*J3^2*J4
     - 688/2460375*J2^3*J3^4*J4 + 32/32805*J3^6*J4 - 
     9233/2214337500*J2^7*J4^2 + 21931/73811250*J2^4*J3^2*J4^2 - 
     12698/2460375*J2*J3^4*J4^2 + 2341231/3444525000*J2^5*J4^3 - 
     2032531/114817500*J2^2*J3^2*J4^3 - 3863621/191362500*J2^3*J4^4 - 
     29713/607500*J3^2*J4^4 + 126187/8859375*J2*J4^5 + 91/27337500*J2^7*J3*J5
     - 91/455625*J2^4*J3^3*J5 + 91/30375*J2*J3^5*J5 - 
     224/1366875*J2^5*J3*J4*J5 + 448/91125*J2^2*J3^3*J4*J5 - 
     17209/38272500*J2^3*J3*J4^2*J5 + 20732/637875*J3^3*J4^2*J5 + 
     69997/8505000*J2*J3*J4^3*J5 + 91/2025000*J2^6*J5^2 - 
     91/67500*J2^3*J3^2*J5^2 - 509/202500*J2^4*J4*J5^2 + 
     19/450*J2*J3^2*J4*J5^2 + 357949/2835000*J2^2*J4^2*J5^2 - 
     1109/26250*J4^3*J5^2 + 91/15000*J2^2*J3*J5^3 - 13/125*J3*J4*J5^3 - 
     819/5000*J2*J5^4 + 16039/2952450000*J2^8*J6 - 
     16039/49207500*J2^5*J3^2*J6 + 16039/3280500*J2^2*J3^4*J6 - 
     219271/382725000*J2^6*J4*J6 + 1916969/114817500*J2^3*J3^2*J4*J6 + 
     5647/382725*J3^4*J4*J6 + 6060043/344452500*J2^4*J4^2*J6 - 
     571733/9185400*J2*J3^2*J4^2*J6 - 2085949/19136250*J2^2*J4^3*J6 + 
     115784/1771875*J4^4*J6 - 2053/3645000*J2^4*J3*J5*J6 + 
     2053/121500*J2*J3^3*J5*J6 - 18607/1215000*J2^2*J3*J4*J5*J6 + 
     1003211/4252500*J3*J4^2*J5*J6 - 123529/4860000*J2^3*J5^2*J6 - 
     3611/10125*J3^2*J5^2*J6 + 320273/945000*J2*J4*J5^2*J6 + 
     941/820125*J2^5*J6^2 - 1882/54675*J2^2*J3^2*J6^2 - 
     274/91125*J2^3*J4*J6^2 + 56926/127575*J3^2*J4*J6^2 - 
     56164/382725*J2*J4^2*J6^2 + 571/2025*J2*J3*J5*J6^2 - 22/135*J5^2*J6^2 - 
     8/820125*J2^6*J3*J7 + 32/54675*J2^3*J3^3*J7 - 32/3645*J3^5*J7 - 
     3281/3645000*J2^4*J3*J4*J7 + 3281/121500*J2*J3^3*J4*J7 + 
     779/5000*J2^2*J3*J4^2*J7 - 97039/472500*J3*J4^3*J7 - 
     811/1215000*J2^5*J5*J7 + 811/40500*J2^2*J3^2*J5*J7 + 
     2407/97200*J2^3*J4*J5*J7 + 61/162*J3^2*J4*J5*J7 + 
     269/15000*J2*J4^2*J5*J7 - 1393/3000*J2*J3*J5^2*J7 + 63/250*J5^3*J7 + 
     283/36450*J2^3*J3*J6*J7 - 283/1215*J3^3*J6*J7 - 
     4757/18900*J2*J3*J4*J6*J7 - 19/120*J2^2*J5*J6*J7 + 77/1350*J4*J5*J6*J7 -
     1/1350*J2^4*J7^2 + 1/45*J2*J3^2*J7^2 + 1/900*J2^2*J4*J7^2 + 
     11/210*J4^2*J7^2 - 1309/32805000*J2^7*J8 + 1309/546750*J2^4*J3^2*J8 - 
     1309/36450*J2*J3^4*J8 + 9539/1215000*J2^5*J4*J8 - 
     9539/40500*J2^2*J3^2*J4*J8 - 164492/455625*J2^3*J4^2*J8 - 
     511/2430*J3^2*J4^2*J8 + 1339/20250*J2*J4^3*J8 - 19/10125*J2^3*J3*J5*J8 +
     38/675*J3^3*J5*J8 + 7841/9000*J2*J3*J4*J5*J8 + 2641/6000*J2^2*J5^2*J8 - 
     1913/1500*J4*J5^2*J8 + 181/72900*J2^4*J6*J8 - 181/2430*J2*J3^2*J6*J8 + 
     1543/4050*J2^2*J4*J6*J8 - 172/405*J4^2*J6*J8 + 113/90*J3*J5*J6*J8 + 
     1/6*J3*J4*J7*J8 + J7^2*J8 - 371/911250*J2^5*J3*J9 + 
     371/30375*J2^2*J3^3*J9 + 1417/60750*J2^3*J3*J4*J9 - 304/2025*J3^3*J4*J9 
     + 2998/16875*J2*J3*J4^2*J9 + 14/10125*J2^4*J5*J9 - 
     1393/6750*J2*J3^2*J5*J9 - 41/1250*J2^2*J4*J5*J9 - 1552/5625*J4^2*J5*J9 +
     331/250*J3*J5^2*J9 - 688/2025*J2^2*J3*J6*J9 - 728/675*J3*J4*J6*J9 + 
     1/75*J2^3*J7*J9 - 14/75*J2*J4*J7*J9 + 524/1366875*J2^6*J10 - 
     983/91125*J2^3*J3^2*J10 - 26/1215*J3^4*J10 - 187/12150*J2^4*J4*J10 - 
     178/2025*J2*J3^2*J4*J10 - 15877/16875*J2^2*J4^2*J10 + 5216/5625*J4^3*J10
     + 371/2250*J2^2*J3*J5*J10 - 562/375*J3*J4*J5*J10 + 323/250*J2*J5^2*J10 +
     1736/6075*J2^3*J6*J10 + 656/405*J3^2*J6*J10 - 304/675*J2*J4*J6*J10 - 
     2/5*J2*J3*J7*J10,
     7/2952450000*J2^10*J3 - 7/32805000*J2^7*J3^3 + 7/1093500*J2^4*J3^5 - 
     7/109350*J2*J3^7 + 163/4723920000*J2^8*J3*J4 - 163/78732000*J2^5*J3^3*J4
     + 163/5248800*J2^2*J3^5*J4 - 5273/204120000*J2^6*J3*J4^2 + 
     62749/61236000*J2^3*J3^3*J4^2 - 3823/510300*J3^5*J4^2 + 
     48761/45360000*J2^4*J3*J4^3 - 58481/3402000*J2*J3^3*J4^3 - 
     68851/2268000*J2^2*J3*J4^4 - 45921/1225000*J3*J4^5 + 
     7/2430000*J2^6*J3^2*J5 - 7/40500*J2^3*J3^4*J5 + 7/2700*J3^6*J5 + 
     1121/262440000*J2^7*J4*J5 - 799/3499200*J2^4*J3^2*J4*J5 + 
     1753/583200*J2*J3^4*J4*J5 - 350351/816480000*J2^5*J4^2*J5 + 
     157361/27216000*J2^2*J3^2*J4^2*J5 + 1638899/158760000*J2^3*J4^3*J5 - 
     38081/1764000*J3^2*J4^3*J5 + 8717/294000*J2*J4^4*J5 + 
     7/180000*J2^5*J3*J5^2 - 7/6000*J2^2*J3^3*J5^2 - 
     2711/2592000*J2^3*J3*J4*J5^2 + 1/27*J3^3*J4*J5^2 + 
     976/7875*J2*J3*J4^2*J5^2 + 21/4000*J2*J3^2*J5^3 - 233/9600*J2^2*J4*J5^3 
     - 297/5600*J4^2*J5^3 - 567/4000*J3*J5^4 + 3701/524880000*J2^7*J3*J6 - 
     3701/8748000*J2^4*J3^3*J6 + 3701/583200*J2*J3^5*J6 - 
     620621/734832000*J2^5*J3*J4*J6 + 620621/24494400*J2^2*J3^3*J4*J6 + 
     9086489/357210000*J2^3*J3*J4^2*J6 - 2443381/47628000*J3^3*J4^2*J6 - 
     243491/3969000*J2*J3*J4^3*J6 - 5017/29160000*J2^6*J5*J6 + 
     227/40500*J2^3*J3^2*J5*J6 - 431/32400*J3^4*J5*J6 + 
     2538449/408240000*J2^4*J4*J5*J6 - 1612963/27216000*J2*J3^2*J4*J5*J6 + 
     2244407/79380000*J2^2*J4^2*J5*J6 + 32257/367500*J4^3*J5*J6 - 
     29447/288000*J2^2*J3*J5^2*J6 + 683/36000*J3*J4*J5^2*J6 + 
     521/8000*J2*J5^3*J6 + 83/72900*J2^4*J3*J6^2 - 83/2430*J2*J3^3*J6^2 + 
     9623/116640*J2^2*J3*J4*J6^2 + 6709/198450*J3*J4^2*J6^2 - 
     221/25920*J2^3*J5*J6^2 + 46/135*J3^2*J5*J6^2 - 5851/56700*J2*J4*J5*J6^2 
     - 127/87480000*J2^8*J7 + 127/1458000*J2^5*J3^2*J7 - 
     127/97200*J2^2*J3^4*J7 + 86269/408240000*J2^6*J4*J7 - 
     37117/4536000*J2^3*J3^2*J4*J7 + 12541/226800*J3^4*J4*J7 - 
     15433/5670000*J2^4*J4^2*J7 + 507083/3024000*J2*J3^2*J4^2*J7 - 
     4231/315000*J2^2*J4^3*J7 - 261/8750*J4^4*J7 - 347/1296000*J2^4*J3*J5*J7 
     + 347/43200*J2*J3^3*J5*J7 + 24487/288000*J2^2*J3*J4*J5*J7 + 
     11041/28000*J3*J4^2*J5*J7 - 11/36000*J2^3*J5^2*J7 - 
     923/2400*J3^2*J5^2*J7 - 411/8000*J2*J4*J5^2*J7 + 259/194400*J2^5*J6*J7 -
     259/6480*J2^2*J3^2*J6*J7 - 6313/226800*J2^3*J4*J6*J7 - 
     4031/15120*J3^2*J4*J6*J7 - 89/2100*J2*J4^2*J6*J7 - 83/576*J2*J3*J5*J6*J7
     - 7/80*J5^2*J6*J7 - 1/720*J2^3*J3*J7^2 + 1/24*J3^3*J7^2 + 
     1/960*J2*J3*J4*J7^2 - 7/194400*J2^6*J3*J8 + 7/3240*J2^3*J3^3*J8 - 
     7/216*J3^5*J8 + 10211/1296000*J2^4*J3*J4*J8 - 10211/43200*J2*J3^3*J4*J8 
     - 3469/7200*J2^2*J3*J4^2*J8 - 42041/42000*J3*J4^3*J8 - 
     127/216000*J2^5*J5*J8 + 127/7200*J2^2*J3^2*J5*J8 - 
     25967/3024000*J2^3*J4*J5*J8 + 49093/50400*J3^2*J4*J5*J8 + 
     5867/42000*J2*J4^2*J5*J8 + 1543/3200*J2*J3*J5^2*J8 - 99/800*J5^3*J8 + 
     1/432*J2^3*J3*J6*J8 - 5/72*J3^3*J6*J8 + 599/1440*J2*J3*J4*J6*J8 + 
     173/960*J2^2*J5*J6*J8 + 949/840*J4*J5*J6*J8 - 1/2880*J2^4*J7*J8 + 
     1/96*J2*J3^2*J7*J8 + 7/480*J2^2*J4*J7*J8 - 3/20*J4^2*J7*J8 - 
     149/11664000*J2^7*J9 - 37/388800*J2^4*J3^2*J9 + 31/2160*J2*J3^4*J9 + 
     13889/13608000*J2^5*J4*J9 + 7397/907200*J2^2*J3^2*J4*J9 - 
     298/7875*J2^3*J4^2*J9 + 833/1800*J3^2*J4^2*J9 - 359/3500*J2*J4^3*J9 - 
     199/28800*J2^3*J3*J5*J9 - 17/120*J3^3*J5*J9 - 997/16800*J2*J3*J4*J5*J9 +
     233/1600*J2^2*J5^2*J9 + 579/1400*J4*J5^2*J9 - 211/16200*J2^4*J6*J9 - 
     29/90*J2*J3^2*J6*J9 - 5797/37800*J2^2*J4*J6*J9 - 202/525*J4^2*J6*J9 + 
     1/32*J2^2*J3*J7*J9 - 7/20*J3*J4*J7*J9 + 67/77760*J2^5*J3*J10 - 
     67/2592*J2^2*J3^3*J10 - 50597/1512000*J2^3*J3*J4*J10 - 
     223/1400*J3^3*J4*J10 - 187/168*J2*J3*J4^2*J10 + 2399/432000*J2^4*J5*J10 
     + 1313/7200*J2*J3^2*J5*J10 - 2747/42000*J2^2*J4*J5*J10 - 
     492/875*J4^2*J5*J10 + 237/200*J3*J5^2*J10 + 77/108*J2^2*J3*J6*J10 + 
     107/70*J3*J4*J6*J10 - 8/15*J2*J5*J6*J10 - 17/1440*J2^3*J7*J10 - 
     7/12*J3^2*J7*J10 + 1/5*J2*J4*J7*J10 + J6*J7*J10,
     41/13286025000*J2^10*J3 - 41/147622500*J2^7*J3^3 + 41/4920750*J2^4*J3^5 - 
     41/492075*J2*J3^7 + 233/1180980000*J2^8*J3*J4 - 
     233/19683000*J2^5*J3^3*J4 + 233/1312200*J2^2*J3^5*J4 - 
     198773/4133430000*J2^6*J3*J4^2 + 246661/137781000*J2^3*J3^3*J4^2 - 
     11972/1148175*J3^5*J4^2 + 1578841/918540000*J2^4*J3*J4^3 - 
     347633/15309000*J2*J3^3*J4^3 - 245459/5103000*J2^2*J3*J4^4 + 
     1178/65625*J3*J4^5 + 41/10935000*J2^6*J3^2*J5 - 41/182250*J2^3*J3^4*J5 +
     41/12150*J3^6*J5 + 49/6561000*J2^7*J4*J5 - 1261/4374000*J2^4*J3^2*J4*J5 
     + 281/145800*J2*J3^4*J4*J5 - 19253/24494400*J2^5*J4^2*J5 + 
     35911/4082400*J2^2*J3^2*J4^2*J5 + 19951/945000*J2^3*J4^3*J5 - 
     86083/1134000*J3^2*J4^3*J5 - 1103/45000*J2*J4^4*J5 + 
     41/810000*J2^5*J3*J5^2 - 41/27000*J2^2*J3^3*J5^2 - 
     373/648000*J2^3*J3*J4*J5^2 + 67/1350*J3^3*J4*J5^2 + 
     124093/756000*J2*J3*J4^2*J5^2 + 41/6000*J2*J3^2*J5^3 - 
     16/375*J2^2*J4*J5^3 + 1033/14000*J4^2*J5^3 - 369/2000*J3*J5^4 + 
     7069/590490000*J2^7*J3*J6 - 7069/9841500*J2^4*J3^3*J6 + 
     7069/656100*J2*J3^5*J6 - 266597/183708000*J2^5*J3*J4*J6 + 
     266597/6123600*J2^2*J3^3*J4*J6 + 5091419/114817500*J2^3*J3*J4^2*J6 - 
     3014377/30618000*J3^3*J4^2*J6 - 32411/255150*J2*J3*J4^3*J6 - 
     10043/32805000*J2^6*J5*J6 + 42433/4374000*J2^3*J3^2*J5*J6 - 
     2261/145800*J3^4*J5*J6 + 3845543/306180000*J2^4*J4*J5*J6 - 
     2819701/20412000*J2*J3^2*J4*J5*J6 - 4489/607500*J2^2*J4^2*J5*J6 - 
     949/15750*J4^3*J5*J6 - 9853/54000*J2^2*J3*J5^2*J6 + 
     216049/378000*J3*J4*J5^2*J6 + 499/4000*J2*J5^3*J6 + 
     2191/1312200*J2^4*J3*J6^2 - 2191/43740*J2*J3^3*J6^2 + 
     10823/68040*J2^2*J3*J4*J6^2 - 1955/5103*J3*J4^2*J6^2 - 
     193/11664*J2^3*J5*J6^2 + 581/1215*J3^2*J5*J6^2 - 
     1073/17010*J2*J4*J5*J6^2 - 167/65610000*J2^8*J7 + 
     167/1093500*J2^5*J3^2*J7 - 167/72900*J2^2*J3^4*J7 + 
     50077/131220000*J2^6*J4*J7 - 30643/2187000*J2^3*J3^2*J4*J7 + 
     11209/145800*J3^4*J4*J7 - 156679/25515000*J2^4*J4^2*J7 + 
     1999247/6804000*J2*J3^2*J4^2*J7 - 6599/1417500*J2^2*J4^3*J7 + 
     19/1750*J4^4*J7 - 19/324000*J2^4*J3*J5*J7 + 19/10800*J2*J3^3*J5*J7 + 
     31717/216000*J2^2*J3*J4*J5*J7 - 773/6000*J3*J4^2*J5*J7 - 
     11/18000*J2^3*J5^2*J7 - 593/1200*J3^2*J5^2*J7 - 389/4000*J2*J4*J5^2*J7 +
     353/145800*J2^5*J6*J7 - 353/4860*J2^2*J3^2*J6*J7 - 
     29809/510300*J2^3*J4*J6*J7 - 3953/68040*J3^2*J4*J6*J7 + 
     6131/56700*J2*J4^2*J6*J7 - 5/24*J2*J3*J5*J6*J7 + 1/120*J5^2*J6*J7 - 
     7/3240*J2^3*J3*J7^2 + 7/108*J3^3*J7^2 + 17/720*J2*J3*J4*J7^2 - 
     41/874800*J2^6*J3*J8 + 41/14580*J2^3*J3^3*J8 - 41/972*J3^5*J8 + 
     33203/2916000*J2^4*J3*J4*J8 - 33203/97200*J2*J3^3*J4*J8 - 
     1564/2025*J2^2*J3*J4^2*J8 - 5311/13500*J3*J4^3*J8 - 
     167/162000*J2^5*J5*J8 + 167/5400*J2^2*J3^2*J5*J8 - 
     167/12960*J2^3*J4*J5*J8 + 12647/10800*J3^2*J4*J5*J8 + 
     917/1800*J2*J4^2*J5*J8 + 149/200*J2*J3*J5^2*J8 - 99/400*J5^3*J8 + 
     43/9720*J2^3*J3*J6*J8 - 43/324*J3^3*J6*J8 + 1889/3240*J2*J3*J4*J6*J8 + 
     73/240*J2^2*J5*J6*J8 - 11/12*J4*J5*J6*J8 - 1/6480*J2^4*J7*J8 + 
     1/216*J2*J3^2*J7*J8 - 1/360*J2^2*J4*J7*J8 - 19/60*J4^2*J7*J8 - 
     317/14580000*J2^7*J9 - 91/486000*J2^4*J3^2*J9 + 17/675*J2*J3^4*J9 + 
     12803/7290000*J2^5*J4*J9 + 7019/486000*J2^2*J3^2*J4*J9 - 
     10841/151875*J2^3*J4^2*J9 + 13409/20250*J3^2*J4^2*J9 + 
     1507/11250*J2*J4^3*J9 - 4409/324000*J2^3*J3*J5*J9 - 529/2700*J3^3*J5*J9 
     - 413/4500*J2*J3*J4*J5*J9 + 32/125*J2^2*J5^2*J9 - 301/500*J4*J5^2*J9 - 
     97/4050*J2^4*J6*J9 - 73/135*J2*J3^2*J6*J9 - 593/4050*J2^2*J4*J6*J9 + 
     44/135*J4^2*J6*J9 + 1/24*J2^2*J3*J7*J9 + 11/90*J3*J4*J7*J9 + J7^2*J9 + 
     29/19440*J2^5*J3*J10 - 29/648*J2^2*J3^3*J10 - 
     30839/486000*J2^3*J3*J4*J10 - 893/8100*J3^3*J4*J10 - 
     3889/2700*J2*J3*J4^2*J10 + 3077/324000*J2^4*J5*J10 + 
     431/1350*J2*J3^2*J5*J10 - 377/3000*J2^2*J4*J5*J10 + 96/125*J4^2*J5*J10 +
     36/25*J3*J5^2*J10 + 34/27*J2^2*J3*J6*J10 - 128/135*J3*J4*J6*J10 - 
     J2*J5*J6*J10 - 19/1080*J2^3*J7*J10 - 13/18*J3^2*J7*J10 - 
     17/90*J2*J4*J7*J10,
     -1/1660753125*J2^12 + 2/36905625*J2^9*J3^2 - 4/2460375*J2^6*J3^4 + 
     8/492075*J2^3*J3^6 + 31/574087500*J2^10*J4 - 
     111043/24800580000*J2^7*J3^2*J4 + 50779/413343000*J2^4*J3^4*J4 - 
     30691/27556200*J2*J3^6*J4 + 13193/24800580000*J2^8*J4^2 + 
     35879/826686000*J2^5*J3^2*J4^2 - 6134/3444525*J2^2*J3^4*J4^2 - 
     197611/1607445000*J2^6*J4^3 + 62879/51438240*J2^3*J3^2*J4^3 - 
     33493/21432600*J3^4*J4^3 + 4151597/1071630000*J2^4*J4^4 + 
     4982911/71442000*J2*J3^2*J4^4 - 29791/33075000*J2^2*J4^5 - 
     618/153125*J4^6 - 1/1366875*J2^8*J3*J5 + 4/91125*J2^5*J3^3*J5 - 
     4/6075*J2^2*J3^5*J5 + 20711/1377810000*J2^6*J3*J4*J5 - 
     54389/91854000*J2^3*J3^3*J4*J5 + 12967/3061800*J3^5*J4*J5 + 
     1321181/612360000*J2^4*J3*J4^2*J5 - 698531/20412000*J2*J3^3*J4^2*J5 - 
     30223/567000*J2^2*J3*J4^3*J5 + 461777/13230000*J3*J4^4*J5 - 
     1/101250*J2^7*J5^2 + 1/3375*J2^4*J3^2*J5^2 + 6683/12757500*J2^5*J4*J5^2 
     - 185401/13608000*J2^2*J3^2*J4*J5^2 - 593911/22680000*J2^3*J4^2*J5^2 - 
     13513/378000*J3^2*J4^2*J5^2 - 277327/8820000*J2*J4^3*J5^2 - 
     1/750*J2^3*J3*J5^3 + 6529/50400*J2*J3*J4*J5^3 + 9/250*J2^2*J5^4 + 
     1023/14000*J4*J5^4 - 869/738112500*J2^9*J6 + 
     24943/393660000*J2^6*J3^2*J6 - 4087/6561000*J2^3*J3^4*J6 - 
     191/29160*J3^6*J6 + 1684507/13778100000*J2^7*J4*J6 - 
     859367/393660000*J2^4*J3^2*J4*J6 - 4091473/91854000*J2*J3^4*J4*J6 - 
     87257509/24111675000*J2^5*J4^2*J6 - 14884333/401861250*J2^2*J3^2*J4^2*J\
     6 + 47775629/2679075000*J2^3*J4^3*J6 + 2803813/119070000*J3^2*J4^3*J6 + 
     638129/74418750*J2*J4^4*J6 + 21149/21870000*J2^5*J3*J5*J6 - 
     21149/729000*J2^2*J3^3*J5*J6 - 8585861/306180000*J2^3*J3*J4*J5*J6 + 
     203731/2916000*J3^3*J4*J5*J6 - 759257/4961250*J2*J3*J4^2*J5*J6 + 
     2383/405000*J2^4*J5^2*J6 + 3871/8640*J2*J3^2*J5^2*J6 - 
     244007/3240000*J2^2*J4*J5^2*J6 - 36577/275625*J4^2*J5^2*J6 - 
     61/3000*J3*J5^3*J6 - 199/820125*J2^6*J6^2 + 3127/437400*J2^3*J3^2*J6^2 +
     19/4860*J3^4*J6^2 + 1693/3827250*J2^4*J4*J6^2 - 
     1474783/3061800*J2*J3^2*J4*J6^2 + 213032/13395375*J2^2*J4^2*J6^2 + 
     120664/1488375*J4^3*J6^2 - 1273/97200*J2^2*J3*J5*J6^2 - 
     127/1575*J3*J4*J5*J6^2 + 119/1350*J2*J5^2*J6^2 + 617/65610000*J2^7*J3*J7
     - 617/1093500*J2^4*J3^3*J7 + 617/72900*J2*J3^5*J7 - 
     38369/43740000*J2^5*J3*J4*J7 + 38369/1458000*J2^2*J3^3*J4*J7 - 
     176243/10206000*J2^3*J3*J4^2*J7 - 152059/1360800*J3^3*J4^2*J7 + 
     119009/2835000*J2*J3*J4^3*J7 + 44/273375*J2^6*J5*J7 - 
     16517/2916000*J2^3*J3^2*J5*J7 + 2437/97200*J3^4*J5*J7 - 
     1603/243000*J2^4*J4*J5*J7 - 267799/648000*J2*J3^2*J4*J5*J7 + 
     8807/315000*J2^2*J4^2*J5*J7 + 3221/105000*J4^3*J5*J7 + 
     311/3000*J2^2*J3*J5^2*J7 + 4829/42000*J3*J4*J5^2*J7 - 99/1000*J2*J5^3*J7
     - 1193/145800*J2^4*J3*J6*J7 + 1193/4860*J2*J3^3*J6*J7 + 
     4709/24300*J2^2*J3*J4*J6*J7 - 45917/793800*J3*J4^2*J6*J7 + 
     757/24300*J2^3*J5*J6*J7 + 769/6480*J3^2*J5*J6*J7 + 
     7363/75600*J2*J4*J5*J6*J7 + 1/12150*J2^5*J7^2 - 1/405*J2^2*J3^2*J7^2 + 
     187/113400*J2^3*J4*J7^2 - 643/15120*J3^2*J4*J7^2 - 
     341/12600*J2*J4^2*J7^2 + 139/16402500*J2^8*J8 - 139/273375*J2^5*J3^2*J8 
     + 139/18225*J2^2*J3^4*J8 - 257731/153090000*J2^6*J4*J8 + 
     965459/20412000*J2^3*J3^2*J4*J8 + 13093/136080*J3^4*J4*J8 + 
     498181/6378750*J2^4*J4^2*J8 + 1364197/1701000*J2*J3^2*J4^2*J8 - 
     3077/236250*J2^2*J4^3*J8 - 1987/39375*J4^4*J8 + 97/32400*J2^4*J3*J5*J8 -
     97/1080*J2*J3^3*J5*J8 - 14251/108000*J2^2*J3*J4*J5*J8 - 
     22387/42000*J3*J4^2*J5*J8 - 2587/27000*J2^3*J5^2*J8 - 
     2437/7200*J3^2*J5^2*J8 + 23047/84000*J2*J4*J5^2*J8 - 11/18225*J2^5*J6*J8
     + 22/1215*J2^2*J3^2*J6*J8 - 3701/48600*J2^3*J4*J6*J8 - 
     25/648*J3^2*J4*J6*J8 + 3533/56700*J2*J4^2*J6*J8 - 191/144*J2*J3*J5*J6*J8
     - 37/60*J5^2*J6*J8 + 1/1296*J2^3*J3*J7*J8 - 5/216*J3^3*J7*J8 - 
     13/360*J2*J3*J4*J7*J8 + 433/2916000*J2^6*J3*J9 - 
     1141/291600*J2^3*J3^3*J9 - 79/4860*J3^5*J9 - 
     174191/17010000*J2^4*J3*J4*J9 + 93407/1134000*J2*J3^3*J4*J9 + 
     32681/189000*J2^2*J3*J4^2*J9 - 40577/236250*J3*J4^3*J9 - 
     74/151875*J2^5*J5*J9 + 26591/324000*J2^2*J3^2*J5*J9 + 
     69289/5670000*J2^3*J4*J5*J9 + 27329/189000*J3^2*J4*J5*J9 + 
     24127/157500*J2*J4^2*J5*J9 - 6359/6000*J2*J3*J5^2*J9 - 117/500*J5^3*J9 +
     557/4050*J2^3*J3*J6*J9 + 103/405*J3^3*J6*J9 + 363/350*J2*J3*J4*J6*J9 - 
     1/810*J2^4*J7*J9 - 23/1080*J2*J3^2*J7*J9 - 211/18900*J2^2*J4*J7*J9 + 
     59/525*J4^2*J7*J9 - 197/2187000*J2^7*J10 + 119/291600*J2^4*J3^2*J10 + 
     223/3240*J2*J3^4*J10 + 20891/5103000*J2^5*J4*J10 + 
     69421/680400*J2^2*J3^2*J4*J10 + 31067/170100*J2^3*J4^2*J10 + 
     6731/28350*J3^2*J4^2*J10 - 3053/23625*J2*J4^3*J10 - 
     809/12960*J2^3*J3*J5*J10 - 163/1080*J3^3*J5*J10 + 
     253/280*J2*J3*J4*J5*J10 - 307/1200*J2^2*J5^2*J10 - 9/350*J4*J5^2*J10 - 
     392/6075*J2^4*J6*J10 - 22/9*J2*J3^2*J6*J10 + 716/14175*J2^2*J4*J6*J10 + 
     328/945*J4^2*J6*J10 + 5/9*J3*J5*J6*J10 + 7/120*J2^2*J3*J7*J10 + 
     421/630*J3*J4*J7*J10 + J7^2*J10
     ];
     
fD4 := x^8 + a6*x^6 + a4*x^4 + a2*x^2 + a0;

"_ D4 Lemma."; "";

"           Direct implication       :", IsLemmaDirectTrue(D4Stratum, fD4);

"           Converse implication     :",
    CheckInDepth select
    IsD4LemmaConverseTrue(true, true) and /* J2 = 0, J3 = 0 */
IsD4LemmaConverseTrue(true, false) and /* J2 = 0, J3 = 1 */
IsD4LemmaConverseTrue(false, false) /* J2 = 1, any J3 */
else
    "not checked";
"";

if CheckInDepth then
"        a4 constraints cancelations :";
  EQs := [Numerator(Evaluate(E, Reverse(ShiodaInvariants(fD4)))) : E in [
      -130977000*J7*J3^2-410791500*J7*J6-10716300*J9*J2^2+4365900*J7*J2^3+150793650*J6*J4*J3+39590775*J6*J5*J2-196465500*J7*J4*J2+10716300*J4^2*J3*J2+1205583750*J8*J5-1157360400*J9*J4+254435580*J5*J4^2+321489000*J10*J3-75014100*J5^2*J3-185220*J5*J2^4+7144200*J4*J3^3+5120010*J5*J4*J2^2-238140*J4*J3*J2^3+5556600*J5*J3^2*J2,
      -11022480*J5*J4*J3+2933280*J4*J3^2*J2+784*J2^6+55339200*J6^2+4408992*J4^3+705600*J3^4-36741600*J9*J3-4286520*J5^2*J2+32315760*J6*J4*J2+36741600*J10*J2-64297800*J7*J5+10054800*J6*J3^2-335160*J6*J2^3-47040*J3^2*J2^3+2812320*J4^2*J2^2-97776*J4*J2^4,
      -793800*J7*J2^3+89302500*J7*J6+26460*J5*J2^4-2381400*J4^2*J3*J2-22027950*J6*J4*J3-11013975*J6*J5*J2+41079150*J7*J4*J2+10716300*J5^2*J3-241116750*J8*J5+257191200*J9*J4-56030940*J5*J4^2+23814000*J7*J3^2-1587600*J4*J3^3-793800*J5*J3^2*J2-476280*J5*J4*J2^2+52920*J4*J3*J2^3,
      -483840*J4*J3^2*J2+2449440*J5*J4*J3-112*J2^6-9072000*J6^2-629856*J4^3-100800*J3^4+612360*J5^2*J2-6713280*J6*J4*J2+9185400*J7*J5-2797200*J6*J3^2+93240*J6*J2^3+6720*J3^2*J2^3-498960*J4^2*J2^2+16128*J4*J2^4+18370800*J8*J4
      ]];

SC := Scheme(AffineSpace(Universe(EQs)), GroebnerBasis(EQs));
IC := IrreducibleComponents(SC);

for i := 1 to #IC do
    Ia := ideal<Universe(EQs) |
	GroebnerBasis(Equations(ReducedSubscheme(IC[i])))>; Qa :=

    quo<Universe(EQs) | Ia>;

    Fa := RingOfFractions(Qa);
    q8, q0, q7, q1, q6, q2, q5, q3, q4 := Explode([Fa.i : i in [1..Rank(Fa)]]);

    z := PolynomialRing(Fa).1;
    fD4 := z^8 + q6*z^6 + q4*z^4 + q2*z^2 + q0;

    printf "              At component ";
    for eqs in GroebnerBasis( Equations(ReducedSubscheme(IC[i]))) do
	printf "%o = 0, ", eqs;
    end for;
    printf "automorphism group is contained in %o;\n",
	IdentifyGroup(GeometricAutomorphismGroupFromShiodaInvariants(ShiodaInvariants(fZ)));
end for;
else
"        a4 constraints cancelations : not checked";
end if;
"";

if CheckInDepth then
"        a0 constraints cancelations :";
  EQs := [Numerator(Evaluate(E, Reverse(ShiodaInvariants(fD4)))) : E in [
      8024365440*J10*J4+530456850*J5^2*J4-472252032*J6*J4^2-4089340080*J9*J5+39293100*J4^2*J3^2-16669800*J5*J3^3-400029840*J6^2*J2-114388848*J4^3*J2-3351600*J3^4*J2-206671500*J10*J2^2+27862380*J5^2*J2^2+3472081200*J7^2-3724*J2^7+36996750*J5*J4*J3*J2+1130212440*J6*J5*J3-382571910*J7*J4*J3+507952620*J7*J5*J2-105019740*J8*J4*J2+206671500*J9*J3*J2-140277690*J6*J3^2*J2-307355310*J6*J4*J2^2-14647500*J4*J3^2*J2^2+555660*J5*J3*J2^3+10835370*J8*J2^3-15739920*J4^2*J2^3+4675923*J6*J2^4+223440*J3^2*J2^4+488250*J4*J2^5-325061100*J8*J3^2-3974055200*J8*J6,
      -529079040*J5*J4*J3+1551156480*J6*J4*J2+140797440*J4*J3^2*J2-2257920*J3^2*J2^3-4693248*J4*J2^4-1763596800*J9*J3+1763596800*J10*J2+2656281600*J6^2+33868800*J3^4+37632*J2^6+211631616*J4^3+134991360*J4^2*J2^2-16087680*J6*J2^3-205752960*J5^2*J2+482630400*J6*J3^2-3086294400*J7*J5,
      -2469035520*J10*J4-96446700*J5^2*J4+62215776*J6*J4^2+1080203040*J9*J5-11736900*J4^2*J3^2+2381400*J5*J3^3+20230560*J6^2*J2+15422724*J4^3*J2+478800*J3^4*J2-3980340*J5^2*J2^2-868020300*J7^2+532*J2^7-10206000*J5*J4*J3*J2-257905620*J6*J5*J3+151099830*J7*J4*J3-72564660*J7*J5*J2-184779630*J8*J4*J2+34838370*J6*J3^2*J2+66418380*J6*J4*J2^2+2457000*J4*J3^2*J2^2-79380*J5*J3*J2^3-1547910*J8*J2^3+2999430*J4^2*J2^3-1161279*J6*J2^4-31920*J3^2*J2^4-81900*J4*J2^5+46437300*J8*J3^2+3815002800*J8*J6,
      117573120*J5*J4*J3-322237440*J6*J4*J2-23224320*J4*J3^2*J2+322560*J3^2*J2^3+774144*J4*J2^4+881798400*J8*J4-435456000*J6^2-4838400*J3^4-5376*J2^6-30233088*J4^3-23950080*J4^2*J2^2+4475520*J6*J2^3+29393280*J5^2*J2-134265600*J6*J3^2+440899200*J7*J5
      ]];

SC := Scheme(AffineSpace(Universe(EQs)), GroebnerBasis(EQs));
IC := IrreducibleComponents(SC);

for i := 1 to #IC do
    Ia := ideal<Universe(EQs) |
	GroebnerBasis(Equations(ReducedSubscheme(IC[i])))>; Qa :=

    quo<Universe(EQs) | Ia>;

    Fa := RingOfFractions(Qa);
    q8, q0, q7, q1, q6, q2, q5, q3, q4 := Explode([Fa.i : i in [1..Rank(Fa)]]);

    z := PolynomialRing(Fa).1;
    fD4 := z^8 + q6*z^6 + q4*z^4 + q2*z^2 + q0;

    printf "              At component ";
    for eqs in GroebnerBasis( Equations(ReducedSubscheme(IC[i]))) do
	printf "%o = 0, ", eqs;
    end for;
    printf "automorphism group is contained in %o;\n",
	IdentifyGroup(GeometricAutomorphismGroupFromShiodaInvariants(ShiodaInvariants(fZ)));
end for;
else
"        a0 constraints cancelations : not checked";
end if;
"";

/* C2
 *****/
"_ C2 Lemma."; "";


Relations := [Evaluate(EQ, [J8, J9, J10]) :
    EQ in ShiodaAlgebraicInvariants([J2, J3, J4, J5, J6, J7] : ratsolve := false)];

AllDets := GroebnerBasis(Relations cat
    [C[2] : C in Genus3Determinants([J2, J3, J4, J5, J6, J7, J8, J9, J10])]);

SomeDets := GroebnerBasis(Relations cat
    [C[2] : C in Genus3Determinants([J2, J3, J4, J5, J6, J7, J8, J9, J10]) | C[1] in [
    [ 1, 2, 3 ], [ 1, 5, 7 ], [ 1, 3, 6 ], [ 1, 2, 9 ], [ 1, 4, 8 ], [ 1, 2, 4 ],
    [1,6,12],[1,3,12],[1,3,13],
    [1,4,11],[2,4,7], [1,3,10],[1,3,7], [1,2,11],
    [1,2,10],[1,4,6], [1,2,8], [1,2,6], [1,2,5]
    ]]);

"         19 determinants are enough :", AllDets eq SomeDets;
"     Det. cancelation eq D4 stratum :", GroebnerBasis(D4Stratum) eq SomeDets;

quit;
